SystemOrganization addCategory: #'GT-Examples'!
SystemOrganization addCategory: 'GT-Examples-Core'!
SystemOrganization addCategory: 'GT-Examples-DEPRECATED'!
SystemOrganization addCategory: 'GT-Examples-Exceptions'!
SystemOrganization addCategory: 'GT-Examples-Testing'!
SystemOrganization addCategory: 'GT-Examples-Tools'!
SystemOrganization addCategory: 'GT-Examples-UI'!
SystemOrganization addCategory: 'GT-Examples-Utils'!

!Dictionary methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 21:38'!
removeKey: aKey thenDo: aBlock
	aBlock value: (self removeKey: aKey ifAbsent: [ ^ self ])! !

!RPackageTag methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/2/2016 20:49'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/15/2015 16:56'!
copyWith: anException
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copyWith: anException);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'AndreiChis 10/15/2015 00:15'!
copyWithAll: aCollection
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copy
			addAll: aCollection;
			yourself);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 17:43'!
signaledBy: anException
	(anException isNil and: [ exceptions isEmptyOrNil not ]) ifTrue: [ 
		^ (GTExampleExpectedExceptionNotRaised new 
			expectedError: self;
			exception: anException;
			yourself) signal ].
	(anException notNil and: [ (exceptions includes: anException class) not ]) ifTrue: [ 
		^ (GTExampleUnexpectedExceptionRaised new
			unexpectedError: anException;
			yourself) signal ]! !

Object subclass: #GTAbstractExampleFactory
	instanceVariableNames: 'cache source providers exampleClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 20:53'!
cache
	^ cache ifNil: [ cache := self defaultCache ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
cache: anObject
	^ cache := anObject! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:34'!
createExample: aCompiledMethod
	| example |
	example := self exampleClass new.
	self initializeFromMethod: aCompiledMethod forExample: example.
	^example! !

!GTAbstractExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 21:47'!
defaultCache
	^ GTExampleOrganizer instance! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:18'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ aBlock value! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
exampleClass
	^ exampleClass ifNil: [ exampleClass := self defaultExampleClass ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
exampleClass: anObject
	exampleClass := anObject! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:18'!
exampleFromMethod: aCompiledMethod do: aBlock
	aBlock value: (self cache exampleAt: aCompiledMethod ifAbsentPut: [ self createExample: aCompiledMethod ])! !

!GTAbstractExampleFactory methodsFor: 'enumeration' stamp: 'StefanReichhart 7/2/2016 20:48'!
exampleMethod: aCompiledMethod do: aBlock
	(self isExampleMethod: aCompiledMethod) ifFalse: [ ^ self ].
	self exampleFromMethod: aCompiledMethod do: aBlock! !

!GTAbstractExampleFactory methodsFor: 'enumeration' stamp: 'StefanReichhart 7/2/2016 20:21'!
exampleMethodsDo: aBlock
	self source theNonMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
	self source theMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
		! !

!GTAbstractExampleFactory methodsFor: 'enumeration' stamp: 'StefanReichhart 7/2/2016 20:21'!
examplesDo: aBlock
	self exampleMethodsDo: [ :method |
		self exampleFromMethod: method do: aBlock ]! !

!GTAbstractExampleFactory methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 20:23'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self examplesDo: [ :example | examples add: example ].
	^ examples! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:54'!
initializeFromMethod: aCompiledMethod forExample: aGTExample
	aGTExample methodClass: aCompiledMethod methodClass.
	aGTExample selector: aCompiledMethod selector.! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:22'!
initializePragmasFromMethod: aCompiledMethod forExample: aGTExample
	aCompiledMethod pragmas do: [ :pragma | 
		(aGTExample class canPerform: pragma keyword) ifTrue: [ 
			[ aGTExample perform: pragma keyword withArguments: pragma arguments ]
				on:Error
				do:[ :exception | 
					aGTExample failures add: (exception asGTExamplePragmaError 
						example: aGTExample;
						pragma: pragma;
						yourself) ] ] ]! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:16'!
initializeSubjectsFromClass: aClass forExample: aGTExample
	aClass gtExamplesSubjects do: [ :subject | 
		[ aGTExample subjects add: subject asGTExampleSubject ]
			on:Error
			do: [ :exception | 
				aGTExample failures add: (GTExampleSubjectError new 
					example: aGTExample;
					subject: subject;
					signaledException: exception;
					yourself) ] ] ! !

!GTAbstractExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 19:18'!
isExampleMethod: aCompiledMethod
	self subclassResponsibility! !

!GTAbstractExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:38'!
isGTExampleMethod: aCompiledMethod
	^ aCompiledMethod isGTExampleMethod! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:41'!
provider
	^ provider ifNil: [ provider := self defaultProvider ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:41'!
provider: anObject
	provider := anObject! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 18:58'!
source
	^ source! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 18:58'!
source: anObject
	source := anObject! !

GTAbstractExampleFactory subclass: #GTExampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 19:16'!
defaultExampleClass
	^ GTExample! !

!GTExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:34'!
initializeFromMethod: aCompiledMethod forExample: aGTExample
	super initializeFromMethod: aCompiledMethod forExample: aGTExample.
	
	self initializeSubjectsFromClass: source forExample: aGTExample.
	self initializePragmasFromMethod: aCompiledMethod forExample: aGTExample.
! !

!GTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:00'!
isExampleMethod: aCompiledMethod
	^ self isGTExampleMethod: aCompiledMethod ! !

GTAbstractExampleFactory subclass: #RTExampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!RTExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 19:54'!
defaultExampleClass
	^ RTExampleSelection! !

!RTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:01'!
isExampleMethod: aCompiledMethod
	^ (self isGTExampleMethod: aCompiledMethod)
		or: [ self isOldStyleRTExampleMethod: aCompiledMethod ]! !

!RTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:01'!
isOldStyleRTExampleMethod: aCompiledMethod
	^ (aCompiledMethod selector beginsWith: 'example')
		and: [ (aCompiledMethod selector endsWith: 'Icon') not ]! !

Object subclass: #GTAbstractExampleSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTAbstractExampleSubject methodsFor: 'converting' stamp: 'StefanReichhart 7/2/2016 19:45'!
asGTExampleSubject
	^ self! !

GTAbstractExampleSubject subclass: #GTExampleClassSubject
	instanceVariableNames: 'subjectClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 19:45'!
exists
	^ self subclassResponsibility! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass
	^ subjectClass! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass: anObject
	subjectClass := anObject! !

GTAbstractExampleSubject subclass: #GTExampleInstanceSubject
	instanceVariableNames: 'instance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance
	^ instance! !

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance: anObject
	instance := anObject! !

GTAbstractExampleSubject subclass: #GTExampleMethodSubject
	instanceVariableNames: 'subjectClass subjectSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass
	^ subjectClass! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass: anObject
	subjectClass := anObject! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectSelector
	^ subjectSelector! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectSelector: anObject
	subjectSelector := anObject! !

Object subclass: #GTExample
	instanceVariableNames: 'methodClass selector label description form exceptions subjects parents children properties after failures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Core'!

!GTExample commentStamp: 'TudorGirba 9/6/2015 11:03' prior: 0!
I represent the concept of an example.

I am constructed out of a method that returns an object. I hold the meta-information about that method and I know how to construct the concrete resulting object that is returned by the method. 

I can depend on other examples, and other examples can depend on me. When I depend on another example, the code that produces me uses the result of the other example to construct me.

The class for which I represent the example is called my subject.

The class that offers the method out of which I am constructed is the provider.

I also know how to handle possible exceptions, which makes me a great candidate for encoding assumptions that are otherwise expressed as classic tests.!

!GTExample class methodsFor: 'examples' stamp: 'StefanReichhart 7/2/2016 18:45'!
gtExampleFromPragma
	<gtExample>
	^ GTExampleFinderOLD new newExampleFromPragma: (self class>>#gtExampleFromPragma) pragmas first! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
= anExample
	^ self class = anExample class 
		and: [ self methodClass = anExample methodClass 
			and: [ self selector = anExample selector ] ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 20:44'!
after
	^ after! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 4/2/2016 18:56'!
after: aSelector
	<gtExamplePragma>
	<description: 'A method, provided by the given selector, to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self methodForSelector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 4/10/2016 18:59'!
afterArguments
	self after ifNil: [ ^ #() ].
	^ self after arguments! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 14:27'!
afterClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method (implemented by a gtExampleProvider) to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self methodForClassNamed: aClassOrClassName selector: aSelector! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/2/2016 19:23'!
afterFinish
	^ afterFinish! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 8/24/2015 21:38'!
allDependencies
	| examples |
	examples := Set new.
	(self traverser
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 8/24/2015 21:35'!
allDependents
	| examples |
	examples := Set new.
	(self traverser reversed
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 8/24/2015 20:54'!
arguments
	^ children asArray! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/24/2015 20:58'!
children
	^ children! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:02'!
childrenExamples
	self deprecated: 'Please use #dependencies instead'.
	^ self dependencies! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/29/2015 11:03'!
classNamed: aClassOrClassName
	^ aClassOrClassName isClass 
		ifTrue: [ aClassOrClassName ]
		ifFalse: [ Smalltalk classNamed: aClassOrClassName asString ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
classNamed: aClassName do: aBlock
	^ self classNamed: aClassName do: aBlock ifNone: [ 
		(GTExampleDeclaredClassMissing new
			example: self;
			classNamed: aClassName;
			yourself) signal ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:04'!
classNamed: aClassName do: aBlock1 ifNone: aBlock2
	^ (self classNamed: aClassName)
		ifNotNil: aBlock1
		ifNil: aBlock2! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 9/2/2015 22:19'!
debug
	^ self debugger result! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
debugger
	^ GTExampleDebugger new 
		example: self;
		yourself! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 8/23/2015 12:36'!
dependencies
	^ self arguments collect: [ :each | each example ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/2/2015 20:54'!
dependenciesOrErrors
	^ self arguments collect: [ :each | 
		[ each example ] 
			on: Error 
			do: [ :exception | exception ] ]! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 8/25/2015 22:01'!
dependents
	^ self parents asArray collect: [ :each | each owner ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 13:41'!
depends: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	children add: (self methodForSelector: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:31'!
dependsAll: aString
	<gtExamplePragma>
	<description: 'This example depends on all given selectors found in the given string. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	self substrings: aString do: [ :each | 
		self depends: aString ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 13:42'!
dependsClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given class and selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	children add: (self methodForClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
description
	^ description! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
description: anObject
	<gtExamplePragma>
	<description: 'A full description of this example'>
	
	description := anObject! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/12/2015 20:29'!
detachFromSystem
	self detachResultFromSystem.
	(self traverser reversed
		action: [ :example | example detachResultFromSystem ];
		onRecursionError: [ :aProcessor :anExample | " ignore and continue " ];
		yourself) value.
	children do: [ :proxy | proxy detachFromSystem ].
	parents do: [ :proxy | proxy detachFromSystem ].
	children := OrderedCollection new.
	parents := Set new.! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/4/2015 13:47'!
detachResultFromSystem
	result ifNil: [ ^ self ].
	result detachFromSystem.
	result := nil.! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 19:58'!
equalsExample: anExampleOrExampleProxy
	^ anExampleOrExampleProxy isGTExample
		ifTrue: [ self class = anExampleOrExampleProxy class and: [ self = anExampleOrExampleProxy ] ]
		ifFalse: [ anExampleOrExampleProxy equalsExample: self ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
evaluator
	^ GTExampleEvaluator new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/2/2015 11:52'!
example
	^ self! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/29/2015 10:58'!
exceptions
	^ exceptions! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:31'!
exists
	^ self methodClass includesSelector: self selector! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:04'!
extractionStrategy
	self deprecated: 'Please define your examples using pragmas'.
	^ self shouldNotImplement! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:01'!
extractionStrategy: anObject
	self deprecated: 'Please define your examples using pragmas'.
	self shouldNotImplement! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 6/17/2015 14:56'!
finder 
	^ finder! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/18/2015 19:40'!
gtDebuggerSourceIn: composite
	<gtDebuggerPresentationOrder: 1>
	self method gtInspectorSourceIn: composite! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:18'!
gtDisplayOn: aStream
	self printOn: aStream.
	aStream nextPutAll: ' ('.
	self gtDisplaySummaryOn: aStream.
	aStream nextPutAll: ')'! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 22:15'!
gtDisplaySourceIn: composite
	self deprecated: 'Please use #gtInspectorSourceIn: instead'.
	^ self gtInspectorSourceIn: composite! !

!GTExample methodsFor: 'ui' stamp: 'AndreiChis 11/11/2015 11:39'!
gtDisplaySummaryOn: aStream.
	self label isEmptyOrNil 
		ifFalse: [ ^ aStream nextPutAll: self label ].
	self description isEmptyOrNil 
		ifFalse: [ ^ aStream nextPutAll: self description ].
	aStream nextPutAll: self method printString! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
gtExample
	<gtExamplePragma>
	<description: 'This method is turned into a gtExample if provided'>
	
	^ self! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 22:14'!
gtExample: anObject
	<toBeDeprecated: 'Please define your example-label (or description) using the pragma #label: or #description:'>
	self label: anObject.
	self gtExample! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:04'!
gtExampleFrom: aSelector
	self deprecated: 'Please define your example-dependencies using the pragma #depends: or #dependsClass:selector: or #dependsAll:'.
	self depends: aSelector! !

!GTExample methodsFor: 'ui' stamp: 'StephaneDucasse 6/16/2016 08:46'!
gtInspectorActionEvaluateResultAndGo
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ :p | p selection: self result returnValue ];
		icon: (Smalltalk ui icons iconNamed: #glamorousRight);
		title: 'Evaluate result and go'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/21/2015 20:32'!
gtInspectorDependenciesIn: composite
	<gtInspectorPresentationOrder: 2>
	self arguments ifEmpty: [ ^ self ].
	^ ((self dependenciesOrErrors ifEmpty: [ ^ self ]) gtInspectorItemsIn: composite)
			title: 'Dependencies'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/21/2015 20:32'!
gtInspectorDependentsIn: composite
	<gtInspectorPresentationOrder: 3>
	self dependents ifEmpty: [ ^ self ].
	^ (self dependents gtInspectorItemsIn: composite)
			title: 'Dependents'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:14'!
gtInspectorSourceIn: composite
	<gtInspectorPresentationOrder: 0>
	^ (self method gtInspectorSourceIn: composite)
		title: 'Source'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/21/2015 20:31'!
gtSpotterDependenciesFor: aStep
	<spotterOrder: 100>
	self arguments ifEmpty: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependencies';
		allCandidates: [ self arguments ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/21/2015 20:31'!
gtSpotterDependentsFor: aStep
	<spotterOrder: 101>
	self dependents ifEmpty: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependents';
		allCandidates: [ self dependents ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 13:58'!
gtSpotterPreviewIn: aComposite
	<spotterPreview: 1>
	^ aComposite pharoMethod
		title: [ self gtDisplayString ]; 
		display: [ :entity | entity method sourceCode ];
		smalltalkClass: [ self method methodClass ];
		entity: self! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 12/5/2015 17:36'!
gtSpotterProviderFor: aStep
	<spotterOrder: 103>
	self provider ifNil: [ ^ self ].
	aStep listProcessor
		title: 'Example Provider';
		items: [ Array with: self provider ];
		itemName: [ :aProvider | aProvider gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 12/5/2015 17:31'!
gtSpotterSourceFor: aStep
	<spotterOrder: 103>
	self source ifNil: [ ^ self ].
	aStep listProcessor
		title: 'Example Source';
		items: [ Array with: self source ];
		itemName: [ :aSource | aSource gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 12/5/2015 17:30'!
gtSpotterSubjectsFor: aStep
	<spotterOrder: 102>
	self subjects ifEmpty: [ ^ self ].
	aStep listProcessor
		title: 'Example Subjects';
		allCandidates: [ self subjects ];
		itemName: [ :subject | subject gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/21/2015 20:34'!
hasDependencies
	^ self arguments notEmpty! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/26/2015 20:57'!
hasDependency: anExample
	(self traverser
		action: [ :example | (example equalsExample: anExample) ifTrue: [ ^ true ] ];
		yourself) value.
	^ false! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 19:59'!
hasDependent
	^ self dependents notEmpty! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/26/2015 20:57'!
hasDependent: anExample
	(self traverser reversed
		action: [ :example | (example equalsExample: anExample) ifTrue: [ ^ true ] ];
		yourself) value.
	^ false! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/21/2015 20:34'!
hasDependents
	^ self dependents notEmpty! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 19:59'!
hasParentExample
	self deprecated: 'Please use #hasDependent instead'.
	^ self hasDependent! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/21/2015 20:33'!
hasRelationships
	^ self hasDependents
		or: [ self hasDependencies ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:31'!
hasSubject: aSubject
	^ self classNamed: aSubject 
		do: [ :class | self subjects includes: class ] 
		ifNone: [ self subjects includes: aSubject ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/13/2016 20:33'!
hasValidAfterArguments
	^ self after isNil
		or: [ self after hasValidArguments
			and: [ (self method variableNodes collect: [ :e | e name ]) includesAll: self after arguments ] ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/29/2015 20:21'!
hasValidArguments
	^ self arguments size = self method numArgs! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/29/2015 20:24'!
hasValidDependencies
	[ self dependencies ]
		on: Error
		do: [ ^ false ].
	^ true! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
hash
	^ (self class hash 
		bitXor: self methodClass hash) 
			bitXor: self selector hash! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/4/2015 16:51'!
icon
	^ form! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
icon: aSelector
	<gtExamplePragma>
	<description: 'This is a shortcut for: #iconThemeSelector:'>
	
	self iconThemeSelector: aSelector! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconBase64: aString
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The string must represent a base64 string'>
	
	form := Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: aString readStream)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconBytes: aByteArray
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The icon form is described by the given byte-array'>
	
	form := (Form
		extent: 16@16
		depth: 32
		fromArray: aByteArray 
		offset: 0@0)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The method described by the given class and selector must return a form-icon'>
	
	self classNamed: aClassOrClassName do: [ :class | 
		form := class perform: aSelector ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
iconSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must return an icon form'>
	
	form := (self provider perform: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
iconSelectorBase64: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must return a base64 string'>
	
	self iconBase64: (self provider perform: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
iconThemeSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must be implemented by the currently active theme'>
	
	form := Smalltalk ui icons perform: aSelector! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/10/2015 19:49'!
ignoreCache
	^ self propertyAt: #noCache ifAbsent: [ false ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/24/2016 17:18'!
ignoreNotification
	^ self propertyAt: #noNotification ifAbsent: [ false ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 10:59'!
ignoreTest
	^ self propertyAt: #noTest ifAbsent: [ false ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 4/24/2016 17:16'!
ignoreTestReport
	^ self propertyAt: #noTestReport ifAbsent: [ false ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/2/2016 19:28'!
initialize
	super initialize.
	
	parents := Set new.
	children := OrderedCollection new.
	subjects := Set new.
	failures := Set new.
	exceptions := ExceptionSet new.
	
	properties := IdentityDictionary new.! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/26/2015 10:44'!
initializeFromFinder: aFinder withPragma: aPragma
	finder := aFinder.
	pragma := aPragma.
	source := aFinder source.
	provider := aFinder provider.	
	subjects := aFinder subjects.
	exceptions := ExceptionSet new.
	children := OrderedCollection new.
	parents := Set new.
	properties := IdentityDictionary new.
	
	aPragma method pragmas do: [ :p |
		(self class canPerform: p keyword) 
			ifTrue: [ self perform: p keyword withArguments: p arguments ] ].! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 12/4/2015 20:11'!
initializeIconWith: aBlock
	<todo: 'come up with something better than an empty & generic exception handler'>
	" We don't want to throw exceptions while examples are being created. We regard those as optional / nice-to-have. "
	[ aBlock value ]
		on: Error
		do: [ :exception | ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:57'!
isGTExample
	^ true! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 12:37'!
isValid
	^ self traverser canProcess! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
label
	^ label! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
label: anObject
	<gtExamplePragma>
	<description: 'A short identifier of the name of this example'>
	
	label := anObject! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:30'!
matchesMethod: aCompiledMethod
	^ aCompiledMethod class = CompiledMethod
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:34'!
method
	^ self methodClass >> self selector! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
methodClass
	^ methodClass! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
methodClass: aClass
	methodClass := aClass! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 13:12'!
methodForClass: aClass named: aClassName selector: aSelector
	^ GTExampleMethod new
		methodClass: aClass;
		methodClassName: aClassName;
		selector: aSelector asSymbol;
		yourself! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/2/2016 18:57'!
methodForClass: aClassOrClassName selector: aSelector
	^ aClassOrClassName isClass 
		ifTrue: [ self methodForProvider: aClassOrClassName gtExamplesProvider selector: aSelector ]
		ifFalse: [ self methodForClassNamed: aClassOrClassName selector: aSelector ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 13:43'!
methodForClassNamed: aClassOrClassName selector: aSelector
	^ self classNamed: aClassOrClassName 
		do: [ :class | self methodForClass: class named: class name selector: aSelector ] 
		ifNone: [ self methodForClass: nil named: aClassOrClassName selector: aSelector ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/10/2016 11:47'!
methodForProvider: aProvider selector: aSelector
	^ GTExampleProviderMethod new
		provider: aProvider;
		selector: aSelector asSymbol;
		yourself! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 4/2/2016 18:53'!
methodForSelector: aSelector
	^ self methodForProvider: self provider selector: aSelector! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 19:58'!
methodReference
	self deprecated: 'The selector #methodReference is obsolete'.
	^ self method asRingDefinition! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 3/28/2016 19:27'!
noCache
	<gtExamplePragma>
	<description: 'Never cache the result/returnValue of this example'>
	
	self propertyAt: #noCache put: true! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/24/2016 17:26'!
noNotification
	<gtExamplePragma>
	<description: 'Do not show any notification for this example'>
	
	self propertyAt: #noNotification put: true! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
noTest
	<gtExamplePragma>
	<description: 'Do not run this example while running SUnit tests'>
	
	self propertyAt: #noTest put: true! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 4/24/2016 17:17'!
noTestReport
	<gtExamplePragma>
	<description: 'Do not run this example while running HDTestReport (not yet implemented)'>
	
	self propertyAt: #noTestReport put: true! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:04'!
parentExample: anObject
	self deprecated: 'Please use #dependencies or #children instead'.
	self shouldNotImplement! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/24/2015 20:58'!
parents
	^ parents! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 6/7/2015 17:57'!
pragma
	^ pragma! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:58'!
propertyAt: aKey
	^ (properties ifNil: [ KeyNotFound signalFor: aKey ])
		at: aKey! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:56'!
propertyAt: aKey ifAbsent: aBlock
	^ (properties ifNil: [ ^ aBlock value ])
		at: aKey ifAbsent: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:57'!
propertyAt: aKey ifAbsentPut: aBlock
	^ (properties ifNil: [ properties := IdentityDictionary new ])
		at: aKey ifAbsentPut: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:56'!
propertyAt: aKey put: aValue
	^ (properties ifNil: [ properties := IdentityDictionary new ])
		at: aKey put: aValue! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:54'!
provider 
	^ self methodClass isMeta
		ifTrue: [ self methodClass theNonMetaClass ]
		ifFalse: [ self methodClass theNonMetaClass new ]! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 4/10/2016 14:57'!
providerClass
	^ self provider class! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 3/28/2016 19:13'!
proxyForClass: aClass selector: aSelector
	" we have to resolve the kind of example again - it could be implemented in #class or in the provider of #class !! "
	^ self proxyForProvider: aClass gtExamplesProvider selector: aSelector! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 3/28/2016 19:13'!
proxyForProvider: aProvider selector: aSelector
	^ GTExampleProxy new
		owner: self;
		provider: aProvider;
		selector: aSelector asSymbol;
		yourself! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 3/28/2016 19:14'!
proxyForSelector: aSelector
	^ self proxyForProvider: self provider selector: aSelector! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
raises: aClassOrClassName
	<gtExamplePragma>
	<description: 'This example is expected to raise the given exception. The example will fail if the exception is not raised.'>
	
	self classNamed: aClassOrClassName do: [ :class | 
		self exceptions add: class ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
raisesAny: aString
	<gtExamplePragma>
	<description: 'This example is expected to raise any of the given exceptions. The example will fail if none of the exceptions is raised.'>
	
	self substrings: aString do: [ :each | 
		self raises: each ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 12:34'!
reinitialize
	| method |
	(method := self method) gtExamplesFactory 
		initializeFromMethod: method forExample: self! !

!GTExample methodsFor: 'public' stamp: 'StefanReichhart 9/12/2015 18:03'!
removeFromSystem
	self finder remove: self ifPresent: [ self detachFromSystem ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 8/24/2015 21:47'!
result
	^ result ifNil: [ self run ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/4/2015 13:46'!
result: aResult
	result := aResult! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 11:41'!
resultCached
	^ result! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 11:51'!
resultCached: aResult
	self ignoreCache ifFalse: [ result := aResult ].
	GTExampleNautilus methodUpdated: self method with: aResult.
	^ aResult! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/2/2015 12:00'!
returnValue
	^ self result returnValue! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 7/2/2016 21:55'!
run
	^ self evaluator result ! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
selector
	^ selector! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
selector: aSelector
	selector := aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 5/13/2016 11:02'!
signalableExceptions
	^ self exceptions copyWithAll: { Halt. Error. Exit. UnhandledError. TestFailure. Deprecation. }! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/26/2015 14:19'!
source
	^ source! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 18:37'!
sourceCode
	^ self method sourceCode! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/9/2015 20:10'!
spotterPreviewIn: aComposite inContext: aSpotter
	^ self gtSpotterPreviewIn: aComposite! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:30'!
subject: aClassOrClassName
	<gtExamplePragma>
	
	self classNamed: aClassOrClassName do: [ :class | 
		self subjects add: class asGTExampleSubject ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/29/2015 11:00'!
subjects
	^ subjects! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:39'!
subjectsAll: aString
	<gtExamplePragma>
	
	self substrings: aString do: [ :each | 
		self subject: each ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:09'!
substrings: aString do: aBlock
	(aString substrings: ' .,;') do: [ :each | 
		aBlock value: each trimBoth ]! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:04'!
title
	self deprecated: 'Please define your examples using pragmas'.
	^ self label! !

!GTExample methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 20:03'!
title: aString
	self deprecated: 'Please define your examples using pragmas'.
	self label: aString! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
traverser
	^ GTExampleTraverser new 
		example: self;
		yourself! !

Object subclass: #GTExampleFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleFinder class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:18'!
allGTExamples
	" self allGTExamples "
	| examples |
	examples := OrderedCollection new.
	self allGTExamplesTo: examples.
	^ examples! !

!GTExampleFinder class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
allGTExamplesTo: examples
	Object withAllSubclasses do: [ :class | 
		class gtExamplesFinder ifNotNil: [ :finder | 
			finder gtExamplesContainedTo: examples ] ]! !

!GTExampleFinder class methodsFor: 'examples' stamp: 'StefanReichhart 9/7/2015 21:42'!
gtExampleInstance
	<gtExample>
	<label: 'An example finder on the source/provider #Number'>
	
	^ self new
		source: Number;
		provider: Number;
		subjects: #();
		yourself! !

!GTExampleFinder class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContained
	" self gtExamplesContained "
	| examples |
	examples := OrderedCollection new.
	self gtExamplesContainedTo: examples.
	^ examples! !

!GTExampleFinder class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContainedTo: examples
	Object withAllSubclasses do: [ :class | 
		class gtExamplesFinder ifNotNil: [ :finder | 
			self = finder class ifTrue: [ 
				finder gtExamplesContainedTo: examples ] ] ]! !

!GTExampleFinder class methodsFor: 'ui' stamp: 'StefanReichhart 12/5/2015 17:40'!
gtSpotterExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleFinder class methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:42'!
myExamples
	self deprecated: 'Please use #gtExamplesContained instead'.
	^ self gtExamplesContained! !

!GTExampleFinder class methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamplesTo: examples
	self deprecated: 'Please use #gtExamplesContainedTo: instead'.
	^ self gtExamplesContainedTo: examples! !

!GTExampleFinder methodsFor: 'private-primitives' stamp: 'StefanReichhart 9/6/2015 19:34'!
class: aClass selector: aSelector method: aMethod pragma: aPragma
	^ GTExampleOrganizer instance atMethod: aMethod ifAbsentPut: [ self newExampleFromPragma: aPragma ]! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 16:59'!
class: aClass selector: aSelector method: aMethod to: examples
	(aMethod pragmas 
		detect: [ :pragma | pragma isGTExamplePragma ]
		ifNone: [ nil ]) ifNotNil: [ :pragma |
			self example: (self class: aClass selector: aSelector method: aMethod pragma: pragma) to: examples ] ! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
class: aClass selector: aSelector to: examples
	(aClass includesSelector: aSelector) ifFalse: [ 
		^ (GTExampleDeclaredMethodMissing new
			methodClass: aClass;
			selector: aSelector;
			yourself) signal ].
	self class: aClass selector: aSelector method: aClass >> aSelector to: examples! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 6/17/2015 12:52'!
class: aClass to: examples
	aClass selectorsAndMethodsDo: [ :selector :method | 
		self class: aClass selector: selector method: method to: examples ] ! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 8/25/2015 20:45'!
example: anExample to: examples
	(examples anySatisfy: [ :each | each equalsExample: anExample ]) ifTrue: [ ^ self ].
	examples add: anExample! !

!GTExampleFinder methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self gtExamplesContainedTo: examples.
	^ examples! !

!GTExampleFinder methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContainedTo: examples
	self class: self provider class to: examples! !

!GTExampleFinder methodsFor: 'ui' stamp: 'StefanReichhart 9/9/2015 23:33'!
gtSpotterExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 6/17/2015 12:52'!
method: aMethod to: examples
	self class: aMethod methodClass selector: aMethod selector method: aMethod to: examples! !

!GTExampleFinder methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamples
	self deprecated: 'Please use #gtExamplesContained instead'.
	^ self gtExamplesContained! !

!GTExampleFinder methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamplesTo: examples
	self deprecated: 'Please use #gtExamplesContainedTo: instead'.
	^ self gtExamplesContainedTo: examples! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 19:49'!
newExample
	^ GTExample new! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 19:49'!
newExampleFromPragma: aPragma
	^ self newExample
		initializeFromFinder: self withPragma: aPragma;
		yourself ! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 9/9/2015 19:05'!
newImpostorPragma: aCompiledMethod
	^ GTExampleImpostorPragma new
		method: aCompiledMethod;
		yourself! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 6/17/2015 12:30'!
provider
	^ provider! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 6/17/2015 12:31'!
provider: anObject
	provider := anObject! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 8/23/2015 12:41'!
proxy: anExampleProxy
	| examples |
	examples := OrderedCollection new.
	self class: anExampleProxy provider class selector: anExampleProxy selector to: examples.
	^ examples first! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 8/29/2015 11:12'!
proxy: anExampleProxy matchesMethod: aCompiledMethod
	^ aCompiledMethod class = CompiledMethod
		and: [ anExampleProxy provider class = aCompiledMethod methodClass
			and: [ anExampleProxy selector = aCompiledMethod selector ] ]! !

!GTExampleFinder methodsFor: 'private' stamp: 'StefanReichhart 8/29/2015 11:09'!
proxyExists: anExampleProxy
	^ anExampleProxy provider class includesSelector: anExampleProxy selector! !

!GTExampleFinder methodsFor: 'private-primitives' stamp: 'StefanReichhart 9/12/2015 17:58'!
remove: anExample ifPresent: aBlock
	^ GTExampleOrganizer instance removeAtMethod: anExample method ifPresent: aBlock! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:28'!
source
	^ source! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:28'!
source: anObject
	source := anObject! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 13:25'!
subjects
	^ subjects ifNil: [ 
		source 
			ifNotNil: [ OrderedCollection with: source ]
			ifNil: [ OrderedCollection new ] ].! !

!GTExampleFinder methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 13:26'!
subjects: anObject
	subjects := anObject ifNotNil: [ anObject asOrderedCollection ]! !

Object subclass: #GTExampleFinderOLD
	instanceVariableNames: 'source provider subjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleFinderOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:18'!
allGTExamples
	" self allGTExamples "
	| examples |
	examples := OrderedCollection new.
	self allGTExamplesTo: examples.
	^ examples! !

!GTExampleFinderOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
allGTExamplesTo: examples
	Object withAllSubclasses do: [ :class | 
		class gtExamplesFinder ifNotNil: [ :finder | 
			finder gtExamplesContainedTo: examples ] ]! !

!GTExampleFinderOLD class methodsFor: 'examples' stamp: 'StefanReichhart 9/7/2015 21:42'!
gtExampleInstance
	<gtExample>
	<label: 'An example finder on the source/provider #Number'>
	
	^ self new
		source: Number;
		provider: Number;
		subjects: #();
		yourself! !

!GTExampleFinderOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContained
	" self gtExamplesContained "
	| examples |
	examples := OrderedCollection new.
	self gtExamplesContainedTo: examples.
	^ examples! !

!GTExampleFinderOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContainedTo: examples
	Object withAllSubclasses do: [ :class | 
		class gtExamplesFinder ifNotNil: [ :finder | 
			self = finder class ifTrue: [ 
				finder gtExamplesContainedTo: examples ] ] ]! !

!GTExampleFinderOLD class methodsFor: 'ui' stamp: 'StefanReichhart 12/5/2015 17:40'!
gtSpotterExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleFinderOLD class methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:42'!
myExamples
	self deprecated: 'Please use #gtExamplesContained instead'.
	^ self gtExamplesContained! !

!GTExampleFinderOLD class methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamplesTo: examples
	self deprecated: 'Please use #gtExamplesContainedTo: instead'.
	^ self gtExamplesContainedTo: examples! !

!GTExampleFinderOLD methodsFor: 'private-primitives' stamp: 'StefanReichhart 7/2/2016 20:57'!
class: aClass selector: aSelector method: aMethod pragma: aPragma
	^ GTExampleOrganizerOLD instance atMethod: aMethod ifAbsentPut: [ self newExampleFromPragma: aPragma ]! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 16:59'!
class: aClass selector: aSelector method: aMethod to: examples
	(aMethod pragmas 
		detect: [ :pragma | pragma isGTExamplePragma ]
		ifNone: [ nil ]) ifNotNil: [ :pragma |
			self example: (self class: aClass selector: aSelector method: aMethod pragma: pragma) to: examples ] ! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
class: aClass selector: aSelector to: examples
	(aClass includesSelector: aSelector) ifFalse: [ 
		^ (GTExampleDeclaredMethodMissing new
			methodClass: aClass;
			selector: aSelector;
			yourself) signal ].
	self class: aClass selector: aSelector method: aClass >> aSelector to: examples! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 6/17/2015 12:52'!
class: aClass to: examples
	aClass selectorsAndMethodsDo: [ :selector :method | 
		self class: aClass selector: selector method: method to: examples ] ! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 8/25/2015 20:45'!
example: anExample to: examples
	(examples anySatisfy: [ :each | each equalsExample: anExample ]) ifTrue: [ ^ self ].
	examples add: anExample! !

!GTExampleFinderOLD methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self gtExamplesContainedTo: examples.
	^ examples! !

!GTExampleFinderOLD methodsFor: 'public' stamp: 'StefanReichhart 9/9/2015 23:34'!
gtExamplesContainedTo: examples
	self class: self provider class to: examples! !

!GTExampleFinderOLD methodsFor: 'ui' stamp: 'StefanReichhart 9/9/2015 23:33'!
gtSpotterExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 6/17/2015 12:52'!
method: aMethod to: examples
	self class: aMethod methodClass selector: aMethod selector method: aMethod to: examples! !

!GTExampleFinderOLD methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamples
	self deprecated: 'Please use #gtExamplesContained instead'.
	^ self gtExamplesContained! !

!GTExampleFinderOLD methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 23:43'!
myExamplesTo: examples
	self deprecated: 'Please use #gtExamplesContainedTo: instead'.
	^ self gtExamplesContainedTo: examples! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 19:49'!
newExample
	^ GTExample new! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 19:49'!
newExampleFromPragma: aPragma
	^ self newExample
		initializeFromFinder: self withPragma: aPragma;
		yourself ! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 9/9/2015 19:05'!
newImpostorPragma: aCompiledMethod
	^ GTExampleImpostorPragma new
		method: aCompiledMethod;
		yourself! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 6/17/2015 12:30'!
provider
	^ provider! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 6/17/2015 12:31'!
provider: anObject
	provider := anObject! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 8/23/2015 12:41'!
proxy: anExampleProxy
	| examples |
	examples := OrderedCollection new.
	self class: anExampleProxy provider class selector: anExampleProxy selector to: examples.
	^ examples first! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 8/29/2015 11:12'!
proxy: anExampleProxy matchesMethod: aCompiledMethod
	^ aCompiledMethod class = CompiledMethod
		and: [ anExampleProxy provider class = aCompiledMethod methodClass
			and: [ anExampleProxy selector = aCompiledMethod selector ] ]! !

!GTExampleFinderOLD methodsFor: 'private' stamp: 'StefanReichhart 8/29/2015 11:09'!
proxyExists: anExampleProxy
	^ anExampleProxy provider class includesSelector: anExampleProxy selector! !

!GTExampleFinderOLD methodsFor: 'private-primitives' stamp: 'StefanReichhart 7/2/2016 20:57'!
remove: anExample ifPresent: aBlock
	^ GTExampleOrganizerOLD instance removeAtMethod: anExample method ifPresent: aBlock! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:28'!
source
	^ source! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:28'!
source: anObject
	source := anObject! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 13:25'!
subjects
	^ subjects ifNil: [ 
		source 
			ifNotNil: [ OrderedCollection with: source ]
			ifNil: [ OrderedCollection new ] ].! !

!GTExampleFinderOLD methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 13:26'!
subjects: anObject
	subjects := anObject ifNotNil: [ anObject asOrderedCollection ]! !

Object subclass: #GTExampleImpostorPragma
	instanceVariableNames: 'method'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleImpostorPragma methodsFor: 'comparing' stamp: 'AlexandreBergel 2/29/2016 09:11'!
= aImpostorPragma
	^ self class == aImpostorPragma class
		and: [ self method == aImpostorPragma method ]! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 18:59'!
arguments
	^ #()! !

!GTExampleImpostorPragma methodsFor: 'comparing' stamp: 'StefanReichhart 9/6/2015 19:30'!
hash
	^ self method hash! !

!GTExampleImpostorPragma methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 19:02'!
isGTExamplePragma
	^ true! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 19:00'!
keyword
	^ #gtExample! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 18:58'!
method
	^ method! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 18:58'!
method: anObject
	method := anObject! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 18:59'!
methodClass
	^ method methodClass! !

!GTExampleImpostorPragma methodsFor: 'accessing-method' stamp: 'StefanReichhart 9/6/2015 18:59'!
selector
	^ method selector! !

Object subclass: #GTExampleMethod
	instanceVariableNames: 'methodClassName methodClass selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:35'!
= anExampleMethod
	^ self class = anExampleMethod class 
		and: [ self methodClassName = anExampleMethod methodClassName 
			and: [ self methodClass = anExampleMethod methodClass
				and: [ self selector = anExampleMethod selector ] ] ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments
	^ arguments ifNil: [ #() ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments: anObject
	arguments := anObject! !

!GTExampleMethod methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 13:27'!
ensure
	methodClass 
		ifNil: [ methodClass := Smalltalk classNamed: self methodClassName ]
		ifNotNil: [ methodClass isObsolete ifTrue: [ methodClass := nil ] ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:36'!
exists
	^ self methodClassName notNil
		and: [ self methodClass notNil
			and: [ self methodClass includesSelector: self selector ] ]! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 13:39'!
gtExample
	^ self hasExample 
		ifTrue: [ self method gtExample ]
		ifFalse: [ nil ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 13:33'!
hasExample
	self exists
		and: [ self method isGTExampleMethod ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:36'!
hasValidArguments
	^ self exists 
		and: [ self method numArgs = self arguments size ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:34'!
hash
	^ ((self class hash 
		bitXor: self methodClassName hash)
			bitXor: self methodClass hash)
				bitXor: self selector hash! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:37'!
matchesMethod: aCompiledMethod
	^ self exists
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 13:29'!
method
	^ self exists 
		ifTrue: [ self methodClass >> self selector ]
		ifFalse: [ nil ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:16'!
methodClass
	^ methodClass! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:04'!
methodClass: anObject
	methodClass := anObject! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:10'!
methodClassName
	^ methodClassName! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:10'!
methodClassName: anObject
	methodClassName := anObject! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:54'!
selector
	^ selector! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:53'!
selector: anObject
	selector := anObject! !

Object subclass: #GTExampleMethodOLD
	instanceVariableNames: 'provider selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleMethodOLD methodsFor: 'testing' stamp: 'StefanReichhart 4/2/2016 19:04'!
= anExampleMethod
	^ self class = anExampleMethod class 
		and: [ self provider = anExampleMethod provider 
			and: [ self selector = anExampleMethod selector ] ]! !

!GTExampleMethodOLD methodsFor: 'testing' stamp: 'StefanReichhart 4/13/2016 20:41'!
exists
	^ self provider notNil
		and: [ self providerClass notNil
			and: [ self providerClass includesSelector: self selector ] ]! !

!GTExampleMethodOLD methodsFor: 'testing' stamp: 'StefanReichhart 4/10/2016 18:28'!
hasValidArguments
	^ true! !

!GTExampleMethodOLD methodsFor: 'public' stamp: 'StefanReichhart 4/2/2016 19:04'!
hash
	^ (self class hash 
		bitXor: self provider hash)
			bitXor: self selector hash! !

!GTExampleMethodOLD methodsFor: 'testing' stamp: 'StefanReichhart 4/2/2016 17:59'!
matchesMethod: aCompiledMethod
	^ self exists
		and: [ self method = aCompiledMethod ]! !

!GTExampleMethodOLD methodsFor: 'public' stamp: 'StefanReichhart 4/2/2016 17:56'!
method
	^ self providerClass >> self selector! !

!GTExampleMethodOLD methodsFor: 'accessing' stamp: 'StefanReichhart 4/10/2016 11:47'!
provider
	^ provider! !

!GTExampleMethodOLD methodsFor: 'accessing' stamp: 'StefanReichhart 4/10/2016 11:47'!
provider: anObject
	provider := anObject! !

!GTExampleMethodOLD methodsFor: 'public' stamp: 'StefanReichhart 4/2/2016 17:55'!
providerClass
	^ self provider class! !

!GTExampleMethodOLD methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:54'!
selector
	^ selector! !

!GTExampleMethodOLD methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:53'!
selector: anObject
	selector := anObject! !

GTExampleMethodOLD subclass: #GTExampleProviderMethod
	instanceVariableNames: 'providerName arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleProviderMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/10/2016 18:59'!
arguments
	^ arguments ifNil: [ #() ]! !

!GTExampleProviderMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/10/2016 11:48'!
arguments: anObject
	arguments := anObject! !

!GTExampleProviderMethod methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 4/13/2016 20:40'!
defaultProvider
	^ [ (Smalltalk classNamed: providerName) gtExamplesProvider ]
		on: Error
		do: [ nil ]! !

!GTExampleProviderMethod methodsFor: 'testing' stamp: 'StefanReichhart 4/13/2016 20:28'!
hasValidArguments
	^ self exists 
		and: [ self arguments size = self method numArgs ]! !

!GTExampleProviderMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/13/2016 20:40'!
provider
	^ provider ifNil: [ provider := self defaultProvider ]! !

!GTExampleProviderMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 18:34'!
provider: anObject
	provider := anObject.
	providerName := nil.! !

!GTExampleProviderMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 18:33'!
providerName: anObject
	providerName := anObject.
	provider := nil.! !

GTExampleMethodOLD subclass: #GTExampleProxy
	instanceVariableNames: 'owner example'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleProxy methodsFor: 'testing' stamp: 'StefanReichhart 4/2/2016 19:03'!
= anExampleProxy
	^ super = anExampleProxy
		and: [ self owner = anExampleProxy owner ]! !

!GTExampleProxy methodsFor: 'private' stamp: 'StefanReichhart 8/25/2015 20:56'!
detachFromSystem
	example ifNotNil: [ example parents remove: self ifAbsent: [ ] ].
	example := nil! !

!GTExampleProxy methodsFor: 'testing' stamp: 'StefanReichhart 8/2/2015 12:08'!
equalsExample: anExampleOrExampleProxy
	^ self withExampleDo: [ anExampleOrExampleProxy equalsExample: self example ] ifAbsent: [ false ] ! !

!GTExampleProxy methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 17:43'!
example
	example ifNil: [ self resolve ].
	^ example! !

!GTExampleProxy methodsFor: 'accessing' stamp: 'StefanReichhart 6/17/2015 13:10'!
example: anObject
	example := anObject! !

!GTExampleProxy methodsFor: 'testing' stamp: 'StefanReichhart 8/25/2015 22:01'!
exists
	^ self owner finder proxyExists: self! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 7/18/2015 19:37'!
gtDebuggerSourceIn: composite
	<gtDebuggerPresentationOrder: 1>
	self withExampleDo: [ self example gtDebuggerSourceIn: composite ]! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 9/2/2015 21:41'!
gtDisplayOn: aStream
	self withExampleDo: [ self example gtDisplayOn: aStream ] ifAbsent: [ 
		aStream nextPutAll: 'anExample ('.
		aStream nextPutAll: self provider class name; nextPutAll: '>>#'; nextPutAll: self selector.
		aStream nextPutAll: ')' ]! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:00'!
gtDisplaySummaryOn: aStream
	self withExampleDo: [ self example gtDisplaySummaryOn: aStream ] ! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:14'!
gtInspectorDependenciesIn: composite
	<gtInspectorPresentationOrder: 1>
	^ self withExampleDo: [ self example gtInspectorDependenciesIn: composite ]! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:14'!
gtInspectorSourceIn: composite
	<gtInspectorPresentationOrder: 0>
	^ self withExampleDo: [ self example gtInspectorSourceIn: composite ]! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/24/2015 21:31'!
gtSpotterDependenciesFor: aStep
	<spotterOrder: 1>
	aStep listProcessor
		title: 'Example Dependencies';
		allCandidates: [ self example arguments ];
		itemName: [ :each | each gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/25/2015 22:10'!
gtSpotterDependentsFor: aStep
	<spotterOrder: 2>
	aStep listProcessor
		title: 'Example Dependents';
		allCandidates: [ self example dependents ];
		itemName: [ :each | each gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleProxy methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 13:58'!
gtSpotterPreviewIn: aComposite
	<spotterPreview: 1>
	self withExampleDo: [ self example gtSpotterPreviewIn: aComposite ]! !

!GTExampleProxy methodsFor: 'public' stamp: 'StefanReichhart 4/2/2016 17:56'!
hash
	^ super hash
		bitXor: self owner hash! !

!GTExampleProxy methodsFor: 'testing' stamp: 'StefanReichhart 9/12/2015 12:17'!
isValid
	^ self withExampleDo: [ self example isValid ] ifAbsent: [ false ]! !

!GTExampleProxy methodsFor: 'testing' stamp: 'StefanReichhart 8/25/2015 22:01'!
matchesMethod: aCompiledMethod
	^ self owner finder proxy: self matchesMethod: aCompiledMethod! !

!GTExampleProxy methodsFor: 'accessing' stamp: 'StefanReichhart 8/25/2015 22:00'!
owner
	^ owner! !

!GTExampleProxy methodsFor: 'accessing' stamp: 'StefanReichhart 8/25/2015 22:00'!
owner: anObject
	owner := anObject! !

!GTExampleProxy methodsFor: 'private' stamp: 'StefanReichhart 8/26/2015 21:03'!
resolve
	example := (self owner finder copy
		subjects: nil; " do not inherit subjects from the owner -> #initializeFromFinder:withPragma: "
		yourself) proxy: self.
	example ifNotNil: [ example parents add: self ]! !

!GTExampleProxy methodsFor: 'private' stamp: 'StefanReichhart 7/18/2015 19:37'!
withExampleDo: aBlock
	^ self withExampleDo: aBlock ifAbsent: [ " ignore " ]! !

!GTExampleProxy methodsFor: 'private' stamp: 'StefanReichhart 7/18/2015 19:37'!
withExampleDo: aBlock ifAbsent: anotherBlock
	^ [ aBlock value ]
		on: Error
		do: anotherBlock! !

Object subclass: #GTExampleNautilus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-UI'!

!GTExampleNautilus class methodsFor: 'private' stamp: 'NicolaiHess 11/22/2015 18:25'!
methodChanged: aCompiledMethod
	[	MethodWidget methodsIconsCache removeKey: aCompiledMethod. 
		NautilusUI allSubInstances do: #updateCategoryAndMethodList ]
			on: Error
			do: [ " ignore and continue " ]! !

!GTExampleNautilus class methodsFor: 'private' stamp: 'StefanReichhart 11/29/2015 11:51'!
methodUpdated: aCompiledMethod with: aResult
	[	MethodWidget methodsIconsCache 
			at: aCompiledMethod 
			put: (GTExampleNautilusAction iconForMethod: aCompiledMethod result: aResult). 
		NautilusUI allSubInstances do: #updateCategoryAndMethodList ]
			on: Error
			do: [ " ignore and continue " ]! !

!GTExampleNautilus methodsFor: 'private' stamp: 'JurajKubelka 11/26/2015 12:13'!
examples: aCollection
	^ aCollection size = 1
		ifTrue: [ aCollection anyOne ]
		ifFalse: [ aCollection ]! !

!GTExampleNautilus methodsFor: 'private' stamp: 'JurajKubelka 11/26/2015 12:11'!
examples: aCollection do: aSelector
	^ aCollection size = 1
		ifTrue: [ aCollection anyOne perform: aSelector ]
		ifFalse: [ aCollection collect: aSelector ]! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 12/5/2015 17:12'!
onClass: aBuilder
	| target class examples |
	target := aBuilder model.
	(class := target selectedClass) ifNil: [ ^ self ].
	self onReferences: class theNonMetaClass name builder: aBuilder.
	self onProvider: class theNonMetaClass builder: aBuilder.
	self onSource: class theNonMetaClass builder: aBuilder.
	(examples := class theNonMetaClass gtExamplesContained) isEmptyOrNil ifTrue: [ ^ self ].
	self onExamples: examples builder: aBuilder! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 9/6/2015 16:37'!
onCompiledMethod: aBuilder
	| target method examples |
	target := aBuilder model.
	(method := target selectedMethod) ifNil: [ ^ self ].
	self onReferences: method selector builder: aBuilder.
	(examples := method gtExamplesContained) isEmptyOrNil ifTrue: [ ^ self ].
	self onExamples: examples builder: aBuilder
! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 1/9/2016 21:03'!
onExamples: aCollection builder: aBuilder
	(aBuilder item: #'Inspect Examples')
		order: 90003;
		action: [ (self examples: aCollection) inspect ].
	
	(aBuilder item: #'Inspect Return-Values')
		order: 90004;
		action: [ (self examples: aCollection do: #returnValue) inspect ].
		
	(aBuilder item: #'Run Examples')
		order: 10004;
		action: [ (self examples: aCollection do: #run) inspect ].
		
	(aBuilder item: #'Debug Examples')
		order: 10005;
		action: [ (self examples: aCollection do: #debug) ].
		
		! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 9/6/2015 16:37'!
onPackage: aBuilder
	| target package examples |
	target := aBuilder model.
	(package := target selectedPackage) ifNil: [ ^ self ].
	package item ifNil: [ ^ self ].
	(examples := package item gtExamplesContained) isEmptyOrNil ifTrue: [ ^ self ].
	self onExamples: examples builder: aBuilder! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 12/5/2015 17:22'!
onProvider: aClass builder: aBuilder
	aClass gtExamplesProvider = aClass ifTrue: [ ^ self ].
	aClass gtExamplesSource = aClass ifTrue: [ ^ self ].
	
	(aBuilder item: #'Browse Example Source ...')
		order: 90002;
		action: [ aClass gtExamplesSource browse ].! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 12/5/2015 17:21'!
onReferences: aLiteral builder: aBuilder
	(aBuilder item: #'Examples of ...')
		order: 90000;
		action: [
			(SystemNavigation default examplesReferencesTo: aLiteral) 
				ifNotEmpty: [ :examples | examples inspect ]
				ifEmpty: [ UIManager default inform: 'No examples found for #', aLiteral asString ] ].
				
! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 12/5/2015 17:21'!
onSource: aClass builder: aBuilder
	aClass gtExamplesProvider = aClass ifTrue: [ ^ self ].
	aClass gtExamplesProvider class = aClass ifTrue: [ ^ self ].
	
	(aBuilder item: #'Browse Example Provider ...')
		order: 90001;
		action: [ aClass gtExamplesProvider browse ].! !

Object subclass: #GTExampleOrganizer
	instanceVariableNames: 'examples theNonMetaClassExamples theMetaClassExamples systemAnnouncements monticelloAnnouncements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

!GTExampleOrganizer class methodsFor: 'examples' stamp: 'StefanReichhart 9/7/2015 21:41'!
gtExampleInstance
	<gtExample>
	<label: 'An instance of an example organizer'>
	
	^ GTExampleOrganizer instance! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
instance
	^ instance ifNil: [ instance := self basicNew initialize start ]! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 9/7/2015 21:42'!
new
	self shouldNotImplement! !

!GTExampleOrganizer class methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:10'!
registerInterestToSystemAnnouncement
	<systemEventRegistration>
	
	self instance 
		stopSystemAnnouncements;
		stopMonticelloAnnouncements;
		startSystemAnnouncements;
		startMonticelloAnnouncements.! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 20:58'!
reset
	instance ifNotNil: [ instance reset ].
	instance := nil! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
restart
	self stop.
	self start.! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
start
	self instance start! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
stop
	instance ifNotNil: [ instance stop ].
	self reset! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:44'!
announcer
	^ SystemAnnouncer uniqueInstance weak! !

!GTExampleOrganizer methodsFor: 'accessing' stamp: 'StefanReichhart 9/12/2015 17:13'!
atMethod: aCompiledMethod ifAbsentPut: aBlock
	<todo: '#theNonMetaClass is not good since we could not put examples to the instance and class-side at once'>
	^ (examples 
		at: aCompiledMethod methodClass theNonMetaClass name 
		ifAbsentPut: [ WeakIdentityKeyDictionary new ]) 
			at: aCompiledMethod selector 
			ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 9/9/2015 21:20'!
cachedExamplesDo: aBlock
	examples do: [ :selectorsAndExamples |
		selectorsAndExamples do: aBlock ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:29'!
categoryRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self reset ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForDependenciesUsing: aCompiledMethod
	[ self cachedExamplesDo: [ :example |
		example arguments do: [ :dependency |
			(dependency exists not and: [ dependency matchesMethod: aCompiledMethod ]) 
				ifTrue: [ self notifyForDependencyUsing: aCompiledMethod example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 1/10/2016 11:01'!
checkForMissingClasses
	[ GTExampleFinder allGTExamples ]
		on: GTExampleDeclaredClassMissing
		do: [ :exception | 
			self notifyForMissingClasses: exception ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingClasses: aCompiledMethod
	[ aCompiledMethod gtExamples ]
		on: GTExampleDeclaredClassMissing
		do: [ :exception | 
			self notifyForMissingClasses: exception ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingDependencies
	[ self cachedExamplesDo: [ :example | 
		example arguments do: [ :dependency | 
			dependency exists 
				ifFalse: [ self notifyForMissingDependency: dependency example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingDependencies: aCompiledMethod
	[ aCompiledMethod gtExamples do: [ :example | 
		example arguments do: [ :dependency | 
			dependency exists 
				ifFalse: [ self notifyForMissingDependency: dependency example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForNumberOfArguments: aCompiledMethod
	[ aCompiledMethod gtExamples do: [ :example | 
		example arguments size = aCompiledMethod numArgs 
			ifFalse: [ self notifyForNumberOfArguments: example ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:32'!
classRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		theNonMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name ifAbsent: [  ].
		theMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name ifAbsent: [  ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:30'!
classRenamed: anAnnouncement
	self withoutExceptionsDo: [ 
		theNonMetaClassExamples removeKey: anAnnouncement oldName asSymbol ifAbsent: [  ].
		theMetaClassExamples removeKey: anAnnouncement oldName asSymbol ifAbsent: [  ] ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 12/20/2015 16:35'!
example: oldExample modified: anAnnouncement
	anAnnouncement method gtExamples do: [ :example |
		self announcer announce: (GTExampleModified new
			oldExample: oldExample;
			example: example;
			yourself) ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 12/20/2015 16:39'!
example: oldExample removed: anAnnouncement
	self announcer announce: (GTExampleRemoved new
		example: oldExample;
		yourself)! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 12/20/2015 16:35'!
exampleAdded: anAnnouncement
	anAnnouncement method gtExamples do: [ :example |
		self announcer announce: (GTExampleAdded new 
			example: example;
			yourself) ]! !

!GTExampleOrganizer methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 21:04'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ self exampleAtClass: aCompiledMethod methodClass selector: aCompiledMethod selector ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/2/2016 21:18'!
exampleAtClass: class selector: selector ifAbsentPut: aBlock
	((class isMeta 
		ifTrue: [ theMetaClassExamples ]
		ifFalse: [ theNonMetaClassExamples ]) 
			at: class name
			ifAbsentPut: [ WeakIdentityKeyDictionary new ]) 
				at: selector 
				ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 2/26/2016 09:21'!
examples
	self deprecated: 'do not use anymore / there is no replacement'.
	^ GTExampleFinder allGTExamples! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/2/2016 21:18'!
examplesDo: aBlock
	theNonMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ].
	theMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/2/2016 21:19'!
examplesOfClass: class do: aBlock
	(theNonMetaClassExamples at: class name ifAbsent: [ nil ]) ifNotNilDo: [ :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: class value: selector value: example ] ].
	(theNonMetaClassExamples at: class name ifAbsent: [ nil ]) ifNotNilDo: [ :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: class value: selector value: example ] ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/2/2016 21:24'!
examplesOfMethod: aCompiledMethod do: aBlock
	aBlock value: aCompiledMethod methodClass name value: aCompiledMethod selector value: (((aCompiledMethod methodClass isMeta 
		ifTrue: [ theMetaClassExamples ]
		ifFalse: [ theNonMetaClassExamples ]) 
			at: aCompiledMethod methodClass name
			ifAbsent: [ ^ self ]) 
				at: aCompiledMethod selector 
				ifAbsent: [ ^ self ])! !

!GTExampleOrganizer methodsFor: 'ui' stamp: 'StefanReichhart 1/10/2016 11:01'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ GTExampleFinder allGTExamples ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleOrganizer methodsFor: 'initializing' stamp: 'StefanReichhart 7/2/2016 21:02'!
initialize
	super initialize.
	
	theNonMetaClassExamples := WeakIdentityKeyDictionary new.
	theMetaClassExamples := WeakIdentityKeyDictionary new.! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 2/26/2016 10:54'!
jobEnded: anAnnouncement
	<deprecated: 'a call of this method means, there are still old job announcements in this image based on this instance. we do nothing here except wiping all previous announcers. we reset/fix this instance to use the new announcers'>
	[ self stopThoroughly; reset; start; loadStopped: anAnnouncement ] on: Error do: [ " ignore " ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 2/26/2016 10:54'!
jobStarted: anAnnouncement
	<deprecated: 'a call of this method means, there are still old job announcements in this image based on this instance. we do nothing here except wiping all previous announcers. we reset/fix this instance to use the new announcers'>
	[ self stopThoroughly; reset; start; loadStarted: anAnnouncement ] on: Error do: [ " ignore " ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 11/29/2015 15:48'!
loadStarted: anAnnouncement
	self withoutExceptionsDo: [
		self stopSystemAnnouncements.
		self resetQuickly " we must get rid of all example objects lying around - this is really important for consistency " ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 11/29/2015 15:00'!
loadStopped: anAnnouncement
	self withoutExceptionsDo: [ 
		self startSystemAnnouncements ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:31'!
methodAdded: anAnnouncement
	self withoutExceptionsDo: [ ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:39'!
methodModified: anAnnouncement
	self withoutExceptionsDo: [ 
		theNonMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name thenDo: [ :methodDict |
			methodDict removeKey: anAnnouncement method selector ifAbsent: [  ] ].
		theMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name thenDo: [ :methodDict |
			methodDict removeKey: anAnnouncement method selector ifAbsent: [  ] ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/2/2016 21:40'!
methodRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		theNonMetaClassExamples removeKey: anAnnouncement oldMethod methodClass theNonMetaClass name thenDo: [ :methodDict |
			methodDict removeKey: anAnnouncement oldMethod selector ifAbsent: [  ] ].
		theMetaClassExamples removeKey: anAnnouncement oldMethod methodClass theNonMetaClass name thenDo: [ :methodDict |
			methodDict removeKey: anAnnouncement oldMethod selector ifAbsent: [  ] ] ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/18/2015 18:53'!
notify: aString exception: anException
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: aString;
		nextPutAll: ': ';
		nextPutAll: anException messageText ])! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForDependencyUsing: aCompiledMethod example: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Obsolete example dependency in' ;
		cr;
		nextPutAll: anExample method methodClass name asString;
		nextPutAll: ' >> #';
		nextPutAll: anExample method selector asString ])! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 7/24/2015 19:00'!
notifyForMissingClasses: anException
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Referenced class missing' ;
		cr;
		nextPutAll: anException classNamed asString ])! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForMissingDependency: aDependency example: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Missing example dependency' ;
		cr;
		nextPutAll: aDependency owner provider class name asString;
		nextPutAll: ' >> #';
		nextPutAll: aDependency selector asString ])! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForNumberOfArguments: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Invalid number of arguments in' ;
		cr;
		nextPutAll: anExample method methodClass name asString;
		nextPutAll: ' >> #';
		nextPutAll: anExample method selector asString ])! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 1/10/2016 10:42'!
notifying
	self deprecated: 'do not use anymore / there is no replacement'.
	^ false! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:46'!
regenerate
	SmalltalkImage current gtExamplesContained! !

!GTExampleOrganizer methodsFor: 'accessing' stamp: 'StefanReichhart 9/12/2015 18:11'!
removeAtMethod: aCompiledMethod ifPresent: aBlock
	(examples 
		at: aCompiledMethod methodClass theNonMetaClass name 
		ifAbsent: [ ^ nil ]) 
			removeKey: aCompiledMethod selector 
			ifAbsent: [ ^ nil ].
	(examples at: aCompiledMethod methodClass theNonMetaClass name) 
		ifEmpty: [ examples removeKey: aCompiledMethod methodClass theNonMetaClass name ].
	aBlock value.
	^ aCompiledMethod! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:02'!
reset
	self initialize! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 15:51'!
resetQuickly
	" we assume the thorough reset was already performed if there are no examples in here "
	examples isEmptyOrNil ifTrue: [ ^ self ].
	self reset! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 2/26/2016 09:17'!
resetResults
	self deprecated: 'do not use anymore / we wipe everything as a temporar solution'.
	self resetResultsOnly! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 11/29/2015 15:50'!
resetResultsOnly
	" this is expensive, but thorough and makes sure the gc will really get rid of all gtexample-result-related objects "
	GTExampleResult allSubInstances do: [ :each | [ each detachFromSystem ] on: Error do: [ " ignore / do not abort " ] ].! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 1/10/2016 10:42'!
running
	self deprecated: 'do not use anymore / there is no replacement'.
	^ true! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
start
	self startMonticelloAnnouncements.
	self startSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
startMonticelloAnnouncements
	monticelloAnnouncements == true ifTrue: [ ^ self ].
	self announcer
		when: MCVersionLoaderStarted send: #loadStarted: to: self;
		when: MCVersionLoaderStopped send: #loadStopped: to: self.
	monticelloAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
startSystemAnnouncements
	systemAnnouncements == true ifTrue: [ ^ self ].
	self announcer
		when: MethodAdded send: #methodAdded: to: self;
		when: MethodModified send: #methodModified: to: self;
		when: MethodRemoved send: #methodRemoved: to: self;
		when: ClassRemoved send: #classRemoved: to: self;
		when: ClassRenamed send: #classRenamed: to: self;
		when: CategoryRemoved send: #categoryRemoved: to: self.
	systemAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stop
	self stopMonticelloAnnouncements.
	self stopSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 2/26/2016 10:56'!
stopJobAnnouncements
	<deprecated: 'a call of this method means, there are still old announcements in this image. we do nothing here except wiping all previous announcers.'>
	Job jobAnnouncer unsubscribeThoroughly: self! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
stopMonticelloAnnouncements
	self announcer 
		unsubscribe: MCVersionLoaderStarted thoroughlyfor: self;
		unsubscribe: MCVersionLoaderStopped thoroughlyfor: self.
	monticelloAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
stopSystemAnnouncements
	self announcer 
		unsubscribe: MethodAdded thoroughlyfor: self;
		unsubscribe: MethodModified thoroughlyfor: self;
		unsubscribe: MethodRemoved thoroughlyfor: self;
		unsubscribe: ClassRemoved thoroughlyfor: self;
		unsubscribe: ClassRenamed thoroughlyfor: self;
		unsubscribe: CategoryRemoved thoroughlyfor: self.
	systemAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stopThoroughly
	self stop.
	
	Announcer allSubInstances do: [ :each | each unsubscribeThoroughly: self ]! !

!GTExampleOrganizer methodsFor: 'DEPRECATED' stamp: 'StefanReichhart 1/10/2016 10:39'!
withCleanupDo: aBlock
	self deprecated: 'use #withoutExceptionsDo: instead'.
	self withoutExceptionsDo: aBlock! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:14'!
withoutExceptionsDo: aBlock
	[ aBlock value ]
		on: Error
		do: [ :exception | " we should not ignore it, but try to continue " ]! !

Object subclass: #GTExampleOrganizerOLD
	instanceVariableNames: 'examples systemAnnouncements monticelloAnnouncements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

GTExampleOrganizerOLD class
	instanceVariableNames: 'instance'!

GTExampleOrganizerOLD class
	instanceVariableNames: 'instance'!

!GTExampleOrganizerOLD class methodsFor: 'examples' stamp: 'StefanReichhart 7/2/2016 20:57'!
gtExampleInstance
	<gtExample>
	<label: 'An instance of an example organizer'>
	
	^ GTExampleOrganizerOLD instance! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/7/2015 21:42'!
instance
	^ instance ifNil: [ instance := self basicNew initialize start ]! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'StefanReichhart 9/7/2015 21:42'!
new
	self shouldNotImplement! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'PavelKrivanek 4/19/2016 15:03'!
registerInterestToSystemAnnouncement
	<systemEventRegistration>
	
	self instance 
		stopSystemAnnouncements;
		stopMonticelloAnnouncements;
		startSystemAnnouncements;
		startMonticelloAnnouncements.! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'StefanReichhart 6/21/2015 17:03'!
restart
	self stop.
	self start.! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'StefanReichhart 6/21/2015 16:21'!
start
	self instance start! !

!GTExampleOrganizerOLD class methodsFor: 'public' stamp: 'StefanReichhart 6/21/2015 16:20'!
stop
	instance ifNotNil: [ instance stop ].
	instance := nil! !

!GTExampleOrganizerOLD methodsFor: 'accessing' stamp: 'PavelKrivanek 4/15/2016 16:39'!
announcer
	^ SystemAnnouncer uniqueInstance weak! !

!GTExampleOrganizerOLD methodsFor: 'accessing' stamp: 'StefanReichhart 9/12/2015 17:13'!
atMethod: aCompiledMethod ifAbsentPut: aBlock
	<todo: '#theNonMetaClass is not good since we could not put examples to the instance and class-side at once'>
	^ (examples 
		at: aCompiledMethod methodClass theNonMetaClass name 
		ifAbsentPut: [ WeakIdentityKeyDictionary new ]) 
			at: aCompiledMethod selector 
			ifAbsentPut: aBlock! !

!GTExampleOrganizerOLD methodsFor: 'private' stamp: 'StefanReichhart 9/9/2015 21:20'!
cachedExamplesDo: aBlock
	examples do: [ :selectorsAndExamples |
		selectorsAndExamples do: aBlock ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:27'!
categoryRemoved: anAnnouncement
	" TODO: example dependencies and/or proxies might be invalid !!!!!! (string/symbol-references) "
	self withoutExceptionsDo: [ 
		examples keys do: [ :className |
			(examples removeKey: className ifAbsent: [ nil ]) ifNotNil: [ :dict | 
				dict do: [ :example | 
					[ example detachFromSystem ] on: Error do: [ " ignore " ].
					"self example: example removed: anAnnouncement" ] ] ].
		self checkForMissingDependencies.
		self checkForMissingClasses ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForDependenciesUsing: aCompiledMethod
	[ self cachedExamplesDo: [ :example |
		example arguments do: [ :dependency |
			(dependency exists not and: [ dependency matchesMethod: aCompiledMethod ]) 
				ifTrue: [ self notifyForDependencyUsing: aCompiledMethod example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 7/2/2016 18:45'!
checkForMissingClasses
	[ GTExampleFinderOLD allGTExamples ]
		on: GTExampleDeclaredClassMissing
		do: [ :exception | 
			self notifyForMissingClasses: exception ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingClasses: aCompiledMethod
	[ aCompiledMethod gtExamples ]
		on: GTExampleDeclaredClassMissing
		do: [ :exception | 
			self notifyForMissingClasses: exception ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingDependencies
	[ self cachedExamplesDo: [ :example | 
		example arguments do: [ :dependency | 
			dependency exists 
				ifFalse: [ self notifyForMissingDependency: dependency example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForMissingDependencies: aCompiledMethod
	[ aCompiledMethod gtExamples do: [ :example | 
		example arguments do: [ :dependency | 
			dependency exists 
				ifFalse: [ self notifyForMissingDependency: dependency example: example ] ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-checking' stamp: 'StefanReichhart 11/29/2015 14:53'!
checkForNumberOfArguments: aCompiledMethod
	[ aCompiledMethod gtExamples do: [ :example | 
		example arguments size = aCompiledMethod numArgs 
			ifFalse: [ self notifyForNumberOfArguments: example ] ] ]
		on: Error
		do: [ :exception | " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:27'!
classRemoved: anAnnouncement
	" TODO: example dependencies and/or proxies might be invalid !!!!!! (string/symbol-references) "
	self withoutExceptionsDo: [ 
		(examples removeKey: anAnnouncement classRemoved theNonMetaClass name ifAbsent: [ nil ]) ifNotNil: [ :dict | 
			dict do: [ :example |
				[ example detachFromSystem ] on: Error do: [ " ignore " ].
				"self example: example removed: anAnnouncement" ].
			self checkForMissingDependencies.
			self checkForMissingClasses ] ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:24'!
classRenamed: anAnnouncement
	" TODO: example dependencies and/or proxies might be invalid !!!!!! (string/symbol-references) "
	self withoutExceptionsDo: [ 
		examples at: anAnnouncement oldName asSymbol ifPresent: [ :oldValue | 
			examples at: anAnnouncement newName asSymbol put: oldValue ].
		self checkForMissingDependencies ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 12/20/2015 16:35'!
example: oldExample modified: anAnnouncement
	anAnnouncement method gtExamples do: [ :example |
		self announcer announce: (GTExampleModified new
			oldExample: oldExample;
			example: example;
			yourself) ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 12/20/2015 16:39'!
example: oldExample removed: anAnnouncement
	self announcer announce: (GTExampleRemoved new
		example: oldExample;
		yourself)! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 12/20/2015 16:35'!
exampleAdded: anAnnouncement
	anAnnouncement method gtExamples do: [ :example |
		self announcer announce: (GTExampleAdded new 
			example: example;
			yourself) ]! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 7/2/2016 18:45'!
examples
	self deprecated: 'do not use anymore / there is no replacement'.
	^ GTExampleFinderOLD allGTExamples! !

!GTExampleOrganizerOLD methodsFor: 'ui' stamp: 'StefanReichhart 7/2/2016 18:45'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ GTExampleFinderOLD allGTExamples ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleOrganizerOLD methodsFor: 'initializing' stamp: 'StefanReichhart 11/29/2015 14:58'!
initialize
	super initialize.
	
	self reset! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 2/26/2016 10:54'!
jobEnded: anAnnouncement
	<deprecated: 'a call of this method means, there are still old job announcements in this image based on this instance. we do nothing here except wiping all previous announcers. we reset/fix this instance to use the new announcers'>
	[ self stopThoroughly; reset; start; loadStopped: anAnnouncement ] on: Error do: [ " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 2/26/2016 10:54'!
jobStarted: anAnnouncement
	<deprecated: 'a call of this method means, there are still old job announcements in this image based on this instance. we do nothing here except wiping all previous announcers. we reset/fix this instance to use the new announcers'>
	[ self stopThoroughly; reset; start; loadStarted: anAnnouncement ] on: Error do: [ " ignore " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 11/29/2015 15:48'!
loadStarted: anAnnouncement
	self withoutExceptionsDo: [
		self stopSystemAnnouncements.
		self resetQuickly " we must get rid of all example objects lying around - this is really important for consistency " ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 11/29/2015 15:00'!
loadStopped: anAnnouncement
	self withoutExceptionsDo: [ 
		self startSystemAnnouncements ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:27'!
methodAdded: anAnnouncement
	self withoutExceptionsDo: [
		"self exampleAdded: anAnnouncement."
		self checkForMissingDependencies: anAnnouncement method.
		self checkForNumberOfArguments: anAnnouncement method.
		self checkForMissingClasses: anAnnouncement method.
		GTExampleNautilus methodChanged: anAnnouncement method ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:27'!
methodModified: anAnnouncement
	self withoutExceptionsDo: [ 
		((examples at: anAnnouncement oldMethod methodClass theNonMetaClass name ifAbsent: [ IdentityDictionary new ])
			removeKey: anAnnouncement oldMethod selector ifAbsent: [ nil ]) ifNotNil: [ :example |
				[ example detachFromSystem ] on: Error do: [ " ignore " ].
				"self example: example modified: anAnnouncement."
				self checkForMissingDependencies: anAnnouncement method.
				self checkForNumberOfArguments: anAnnouncement method.
				self checkForMissingClasses: anAnnouncement method.
				self checkForDependenciesUsing: anAnnouncement method.
				GTExampleNautilus methodChanged: anAnnouncement method.
				GTExampleNautilus methodChanged: anAnnouncement oldMethod ] ]! !

!GTExampleOrganizerOLD methodsFor: 'private-events' stamp: 'StefanReichhart 1/10/2016 11:27'!
methodRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		((examples at: anAnnouncement method methodClass theNonMetaClass name ifAbsent: [ IdentityDictionary new ])
			removeKey: anAnnouncement method selector ifAbsent: [ nil ]) ifNotNil: [ :example |
				[ example detachFromSystem ] on: Error do: [ " ignore " ].
				"self example: example removed: anAnnouncement."
				self checkForDependenciesUsing: anAnnouncement method.
				GTExampleNautilus methodChanged: anAnnouncement method.  ] ]! !

!GTExampleOrganizerOLD methodsFor: 'private-notifying' stamp: 'StefanReichhart 7/18/2015 18:53'!
notify: aString exception: anException
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: aString;
		nextPutAll: ': ';
		nextPutAll: anException messageText ])! !

!GTExampleOrganizerOLD methodsFor: 'private-notifying' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForDependencyUsing: aCompiledMethod example: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Obsolete example dependency in' ;
		cr;
		nextPutAll: anExample method methodClass name asString;
		nextPutAll: ' >> #';
		nextPutAll: anExample method selector asString ])! !

!GTExampleOrganizerOLD methodsFor: 'private-notifying' stamp: 'StefanReichhart 7/24/2015 19:00'!
notifyForMissingClasses: anException
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Referenced class missing' ;
		cr;
		nextPutAll: anException classNamed asString ])! !

!GTExampleOrganizerOLD methodsFor: 'private-notifying' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForMissingDependency: aDependency example: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Missing example dependency' ;
		cr;
		nextPutAll: aDependency owner provider class name asString;
		nextPutAll: ' >> #';
		nextPutAll: aDependency selector asString ])! !

!GTExampleOrganizerOLD methodsFor: 'private-notifying' stamp: 'StefanReichhart 4/24/2016 17:19'!
notifyForNumberOfArguments: anExample
	anExample ignoreNotification ifTrue: [ ^ self ].
	UIManager default inform: (String streamContents: [ :stream | stream 
		nextPutAll: 'Invalid number of arguments in' ;
		cr;
		nextPutAll: anExample method methodClass name asString;
		nextPutAll: ' >> #';
		nextPutAll: anExample method selector asString ])! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 1/10/2016 10:42'!
notifying
	self deprecated: 'do not use anymore / there is no replacement'.
	^ false! !

!GTExampleOrganizerOLD methodsFor: 'accessing' stamp: 'StefanReichhart 9/12/2015 18:11'!
removeAtMethod: aCompiledMethod ifPresent: aBlock
	(examples 
		at: aCompiledMethod methodClass theNonMetaClass name 
		ifAbsent: [ ^ nil ]) 
			removeKey: aCompiledMethod selector 
			ifAbsent: [ ^ nil ].
	(examples at: aCompiledMethod methodClass theNonMetaClass name) 
		ifEmpty: [ examples removeKey: aCompiledMethod methodClass theNonMetaClass name ].
	aBlock value.
	^ aCompiledMethod! !

!GTExampleOrganizerOLD methodsFor: 'initializing' stamp: 'StefanReichhart 11/29/2015 15:50'!
reset
	" this is expensive, but thorough and makes sure the gc will really get rid of all gtexample-related objects "
	examples := WeakIdentityKeyDictionary new.
	GTExample allSubInstances do: [ :each | [ each detachFromSystem ] on: Error do: [ " ignore / do not abort " ] ].
	GTExampleProxy allSubInstances do: [ :each | [ each detachFromSystem ] on: Error do: [ " ignore / do not abort " ] ].
	self resetResultsOnly.! !

!GTExampleOrganizerOLD methodsFor: 'initializing' stamp: 'StefanReichhart 11/29/2015 15:51'!
resetQuickly
	" we assume the thorough reset was already performed if there are no examples in here "
	examples isEmptyOrNil ifTrue: [ ^ self ].
	self reset! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 2/26/2016 09:17'!
resetResults
	self deprecated: 'do not use anymore / we wipe everything as a temporar solution'.
	self resetResultsOnly! !

!GTExampleOrganizerOLD methodsFor: 'initializing' stamp: 'StefanReichhart 11/29/2015 15:50'!
resetResultsOnly
	" this is expensive, but thorough and makes sure the gc will really get rid of all gtexample-result-related objects "
	GTExampleResult allSubInstances do: [ :each | [ each detachFromSystem ] on: Error do: [ " ignore / do not abort " ] ].! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 1/10/2016 10:42'!
running
	self deprecated: 'do not use anymore / there is no replacement'.
	^ true! !

!GTExampleOrganizerOLD methodsFor: 'public' stamp: 'StefanReichhart 11/29/2015 14:48'!
start
	self startMonticelloAnnouncements.
	self startSystemAnnouncements! !

!GTExampleOrganizerOLD methodsFor: 'private-announcers' stamp: 'AndreiChis 4/18/2016 17:18'!
startMonticelloAnnouncements
	monticelloAnnouncements == true ifTrue: [ ^ self ]. " handle: nil | false | true / avoid multiple subscriptions "
	self announcer
		when: MCVersionLoaderStarted send: #loadStarted: to: self;
		when: MCVersionLoaderStopped send: #loadStopped: to: self.
	monticelloAnnouncements := true! !

!GTExampleOrganizerOLD methodsFor: 'private-announcers' stamp: 'AndreiChis 4/18/2016 17:17'!
startSystemAnnouncements
	systemAnnouncements == true ifTrue: [ ^ self ]. " handle: nil | false | true / avoid multiple subscriptions "
	self announcer
		when: MethodAdded send: #methodAdded: to: self;
		when: MethodModified send: #methodModified: to: self;
		when: MethodRemoved send: #methodRemoved: to: self;
		when: ClassRemoved send: #classRemoved: to: self;
		when: ClassRenamed send: #classRenamed: to: self;
		when: CategoryRemoved send: #categoryRemoved: to: self.
	systemAnnouncements := true! !

!GTExampleOrganizerOLD methodsFor: 'public' stamp: 'StefanReichhart 2/26/2016 09:23'!
stop
	self stopJobAnnouncements. " get rid of obsolete annoucements - temporary solution only !! "
	self stopMonticelloAnnouncements.
	self stopSystemAnnouncements! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 2/26/2016 10:56'!
stopJobAnnouncements
	<deprecated: 'a call of this method means, there are still old announcements in this image. we do nothing here except wiping all previous announcers.'>
	Job jobAnnouncer unsubscribeThoroughly: self! !

!GTExampleOrganizerOLD methodsFor: 'private-announcers' stamp: 'StefanReichhart 2/26/2016 09:49'!
stopMonticelloAnnouncements
	self announcer 
		unsubscribe: MCVersionLoaderStarted thoroughlyfor: self;
		unsubscribe: MCVersionLoaderStopped thoroughlyfor: self.
	monticelloAnnouncements := false! !

!GTExampleOrganizerOLD methodsFor: 'private-announcers' stamp: 'StefanReichhart 2/26/2016 09:50'!
stopSystemAnnouncements
	self announcer 
		unsubscribe: MethodAdded thoroughlyfor: self;
		unsubscribe: MethodModified thoroughlyfor: self;
		unsubscribe: MethodRemoved thoroughlyfor: self;
		unsubscribe: ClassRemoved thoroughlyfor: self;
		unsubscribe: ClassRenamed thoroughlyfor: self;
		unsubscribe: CategoryRemoved thoroughlyfor: self.
	systemAnnouncements := false! !

!GTExampleOrganizerOLD methodsFor: 'public' stamp: 'StefanReichhart 2/26/2016 09:47'!
stopThoroughly
	self stop.
	
	Announcer allSubInstances do: [ :each | each unsubscribeThoroughly: self ]! !

!GTExampleOrganizerOLD methodsFor: 'OBSOLETE' stamp: 'StefanReichhart 1/10/2016 10:39'!
withCleanupDo: aBlock
	self deprecated: 'use #withoutExceptionsDo: instead'.
	self withoutExceptionsDo: aBlock! !

!GTExampleOrganizerOLD methodsFor: 'private' stamp: 'StefanReichhart 11/29/2015 14:56'!
withoutExceptionsDo: aBlock
	[ aBlock value ]
		on: Error
		do: [ :exception | " we should not ignore it, but try to continue " ]! !

Object subclass: #GTExampleProcessor
	instanceVariableNames: 'example context processing onProcess onRecursionError onDependencyError onArgumentError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Tools'!

GTExampleProcessor subclass: #GTExampleEvaluator
	instanceVariableNames: 'processed result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Tools'!

GTExampleEvaluator subclass: #GTExampleDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Tools'!

!GTExampleDebugger methodsFor: 'private' stamp: 'StefanReichhart 7/13/2015 12:14'!
do: aBlock on: anException do: anotherBlock
	aBlock value! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 7/13/2015 12:14'!
do: aBlock on: anException do: anotherBlock
	aBlock
		on: anException
		do: anotherBlock! !

!GTExampleEvaluator methodsFor: 'initializing' stamp: 'StefanReichhart 5/13/2016 11:04'!
initialize
	super initialize.
	
	processed := OrderedCollection new! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 5/13/2016 11:38'!
newResultFor: anExample
	^ GTExampleResult new 
		example: anExample;
		context: self context;
		yourself.! !

!GTExampleEvaluator methodsFor: 'primitives' stamp: 'StefanReichhart 5/13/2016 10:31'!
primitiveProcessExample: anExample withArguments: arguments
	^ anExample provider perform: anExample method selector withArguments: arguments! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 5/13/2016 11:38'!
process: anExample withArguments: aBlock
	| currentResult arguments |
	currentResult := self newResultFor: anExample.
	arguments := nil.
	anExample hasValidArguments 
		ifFalse: [ ^ self onArgumentError value: self value: anExample value: arguments ].
	[ arguments := aBlock value ]
		on: Error
		do: [ :exception | ^ self onDependencyError value: self value: anExample value: exception ].
	self do: [ currentResult returnValue: (self primitiveProcessExample: anExample withArguments: arguments) ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	processed add: currentResult.
	self do: [ self processAfter: anExample ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	anExample == self example ifTrue: [ result := currentResult ].
	anExample exceptions signaledBy: currentResult expectedError.
	^ currentResult returnValue! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 6/24/2016 11:52'!
processAfter: anExample
	anExample == self example ifFalse: [ ^ self ].
	processed reverseDo: [ :processedResult |
		self processAfter: processedResult example withResult: processedResult returnValue ]! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 6/24/2016 12:17'!
processAfter: anExample withResult: anObject
	anExample after ifNil: [ ^ self ].
	anExample after method
		valueWithReceiver: anExample after provider 
		possibleArguments: { anObject }! !

!GTExampleEvaluator methodsFor: 'public' stamp: 'StefanReichhart 5/13/2016 11:38'!
result
	result := self newResultFor: self example.
	self do: [ self value ] 
		on: Error
		do: [ :exception | result returnValue: nil; unexpectedError: exception ].
	^ result! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 8/26/2015 21:24'!
basicProcess: anExample 
	^ self 
		process: anExample 
		withArguments: [ anExample arguments collect: [ :each | self process: each example ] ] ! !

!GTExampleProcessor methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:44'!
canProcess
	[ self value ]
		on: GTExampleError
		do: [ ^ false ].
	^ true! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context
	^ context! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context: anObject
	^ context := anObject! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnArgumentError
	^ [ :aProcessor :anExample :someArguments | 
		aProcessor handleArgumentErrorFor: anExample withArguments: someArguments.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnDependencyError
	^ [ :aProcessor :anExample :anException | 
		aProcessor handleDependencyErrorFor: anExample withException: anException.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 9/2/2015 20:58'!
defaultOnProcess
	^ [ :aProcessor :anExample | 
		aProcessor basicProcess: anExample ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnRecursionError
	^ [ :aProcessor :anExample | 
		aProcessor handleRecursionErrorFor: anExample.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example
	^ example! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example: anObject
	example := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
handleArgumentErrorFor: anExample withArguments: arguments
	^ (GTExampleArgumentError new
		example: anExample;
		expected: anExample method argumentNames;
		defined: anExample arguments;
		evaluated: arguments;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:43'!
handleDependencyErrorFor: anExample withException: anException
	^ (GTExampleDependencyError new
		example: anExample;
		exception: anException;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:43'!
handleRecursionErrorFor: anExample
	^ (GTExampleRecursionDetected new 
		example: self example;
		resolved: processing;
		next: anExample;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'initializing' stamp: 'StefanReichhart 3/28/2016 20:17'!
initialize
	super initialize.
	
	processing := IdentitySet new.
	context := GTExampleContext new! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/29/2015 19:17'!
onArgumentError
	^ onArgumentError ifNil: [ onArgumentError := self defaultOnArgumentError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/29/2015 19:17'!
onArgumentError: anObject
	onArgumentError := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/27/2015 14:29'!
onDependencyError
	^ onDependencyError ifNil: [ onDependencyError := self defaultOnDependencyError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/27/2015 14:29'!
onDependencyError: anObject
	onDependencyError := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess
	^ onProcess ifNil: [ onProcess := self defaultOnProcess ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess: anObject
	onProcess := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 16:02'!
onRecursionError
	^ onRecursionError ifNil: [ onRecursionError := self defaultOnRecursionError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:57'!
onRecursionError: anObject
	onRecursionError := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 8/23/2015 15:23'!
process: anExample
	^ self process: anExample withoutLoopsDo: [ 
		self onProcess value: self value: anExample ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 16:55'!
process: anExample withArguments: aBlock
	self subclassResponsibility! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 8/2/2015 12:52'!
process: anExample withoutLoopsDo: aBlock
	| value |
	" check whether we already scheduled the given example for processing -> prevent recursion "
	value := nil.
	processing do: [ :processingExample |
		(processingExample equalsExample: anExample) ifTrue: [
			^ self onRecursionError value: self value: anExample ] ].	
	processing add: anExample. " prepare processing this example "
	value := aBlock value. " process this example "
	processing remove: anExample. " done processing "
	^ value! !

!GTExampleProcessor methodsFor: 'public' stamp: 'StefanReichhart 3/28/2016 18:55'!
value 
	^ self withContextDo: [ self process: self example ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 3/28/2016 20:17'!
withContextDo: aBlock
	GTCurrentExampleContext use: context during: aBlock! !

GTExampleProcessor subclass: #GTExampleTraverser
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Tools'!

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action
	^ action! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action: anObject
	action := anObject! !

!GTExampleTraverser methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:11'!
process: anExample withArguments: aBlock
	| arguments |
	arguments := nil.
	anExample hasValidArguments 
		ifFalse: [ ^ self onArgumentError value: self value: anExample value: arguments ].
	[ arguments := aBlock value ]
		on: Error
		do: [ :exception | ^ self onDependencyError value: self value: anExample value: exception ].
	self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ]! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 8/29/2015 18:53'!
reversed
	self onProcess: [ :processor :anExample | 
		self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ].
		anExample dependents do: [ :each | processor process: each ] ]! !

Object subclass: #GTExampleResult
	instanceVariableNames: 'example context returnValue expectedError unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context
	^ context! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context: anObject
	context := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
debug
	self isSuccess ifTrue: [ ^ self ].
	^ self example debug! !

!GTExampleResult methodsFor: 'private' stamp: 'StefanReichhart 9/12/2015 20:32'!
detachFromSystem
	example ifNotNil: [ 
		example result: nil.
		GTExampleNautilus methodChanged: example method ].
	example := returnValue := expectedError := unexpectedError := nil.! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example
	^ example! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example: anObject
	example := anObject! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError
	^ expectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError: anObject
	expectedError := anObject! !

!GTExampleResult methodsFor: 'ui' stamp: 'StefanReichhart 4/10/2016 11:09'!
gtDisplayOn: aStream
	self example gtDisplayOn: aStream. 
	aStream nextPutAll: ' --> '.
	self unexpectedError 
		ifNotNil: [ self unexpectedError gtDisplayOn: aStream ]
		ifNil: [ self returnValue gtDisplayOn: aStream ].
! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isError
	^ self isSuccess not
		and: [ self unexpectedError isExampleError ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:53'!
isExpectedError
	^ self expectedError notNil! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isFailure
	^ self isSuccess not
		and: [ self unexpectedError isExampleError not ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isSuccess
	^ self unexpectedError isNil! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:54'!
isUnexpectedError
	^ self unexpectedError notNil! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue
	^ returnValue! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue: anObject
	returnValue := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
run
	^ self example run! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
unexpectedError
	^ unexpectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:47'!
unexpectedError: anObject
	unexpectedError := anObject! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:40'!
asGTExampleSubject
	^ GTExampleClassSubject new 
		subjectClass: self;
		yourself! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:53'!
gtExampleContext
	^ GTCurrentExampleContext value! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:35'!
gtExamplesSubjects
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:42'!
asGTExampleSubject
	^ GTExampleInstanceSubject new 
		instance: self;
		yourself! !

!Object methodsFor: '*GT-Examples-Core-private' stamp: 'StefanReichhart 8/25/2015 20:56'!
detachFromSystem! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/21/2015 19:58'!
equalsExample: anExampleOrExampleFutureOrSomethingElse
	^ false! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:52'!
gtExampleContext
	^ self class gtExampleContext! !

!Object methodsFor: '*GT-Examples-UI' stamp: 'TudorGirba 1/27/2016 00:18'!
gtExampleSourceIn: composite inContext: aGTInspector
	<gtInspectorPresentationOrder: 100>
	| previousRawSelection |
	"we first check to see if we have at least two panes, because only
	if we have two panes does it make sense to check the raw selection of the previous pane"
	((aGTInspector isKindOf: GTInspector) and: [aGTInspector first panes size >= 2]) ifFalse: [ ^ self ].
	
	previousRawSelection := (aGTInspector first panes reverse second port: #rawSelection) value.
	previousRawSelection isGTExample ifTrue: [ 
		previousRawSelection gtInspectorSourceIn: composite ]! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:32'!
gtExamples
	^ self class gtExamples! !

!Object methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/2/2016 20:31'!
gtExamplesContained
	^ self gtExamples! !

!Object methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/2/2016 20:50'!
gtExamplesFactory
	^ self class gtExamplesFactory! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:36'!
gtExamplesSubjects
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'AndreiChis 10/30/2014 14:24'!
isGTExample
	
	^ false! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:33'!
isGTExampleMethod
	^ false! !

!WeakSubscriptionBuilder methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 4/18/2016 20:55'!
unsubscribe: anAnnouncementClass thoroughlyfor: aSubscriber
	announcer unsubscribe: anAnnouncementClass thoroughlyfor: aSubscriber! !

!TraitDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:45'!
gtExamplesFinder
	^ GTExampleFinderOLD new
		source: self gtExamplesSource;
		provider: self gtExamplesProvider;
		subjects: self gtExamplesSubjects;
		yourself! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:51'!
examplesDo: aBlock
	Object withAllSubclassesDo: [ :class | class examplesDo: aBlock ]! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:29'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	Object withAllSubclasses do: [ :class |
		class gtExamplesFactory examplesDo: [ :example | 
			examples add: example ] ].
	^ examples! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:53'!
gtExamplesDo: aBlock
	Object withAllSubclassesDo: [ :class | class gtExamplesDo: aBlock ]! !

Notification subclass: #GTCurrentExampleContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTCurrentExampleContext class methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:45'!
defaultValue
	^ nil! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
use: anObject during: aBlock
	^ aBlock 
		on: self 
		do: [ :notification | notification resume: anObject ]! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
value
	^ self signal! !

!GTCurrentExampleContext methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:44'!
defaultAction
	^ self class defaultValue! !

!RPackage methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/2/2016 20:49'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

!RPackage methodsFor: '*GT-Examples-Tools' stamp: 'StefanReichhart 9/26/2015 10:16'!
matchesPackagePattern: aStringOrPattern
	^ self packageName = aStringOrPattern
		or: [ (aStringOrPattern match: self packageName) 
			or: [ [ self packageName matchesRegexIgnoringCase: aStringOrPattern ] 
						on: Error 
						do: [ false ] ] ]! !

IdentityDictionary subclass: #GTExampleContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 2/26/2016 10:41'!
unsubscribe: anAnnouncementClass thoroughlyfor: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! "
			 and: [ subscription announcementClass == anAnnouncementClass ] ] ]
! !

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 2/26/2016 10:41'!
unsubscribeThoroughly: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! " ] ]
! !

Announcer subclass: #GTExampleAnnouncer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleAnnouncer class
	instanceVariableNames: 'instance'!

GTExampleAnnouncer class
	instanceVariableNames: 'instance'!

!GTExampleAnnouncer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:14'!
instance
	^ instance ifNil: [ instance := self basicNew initialize ]! !

!SubscriptionRegistry methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 11/29/2015 15:27'!
subscriptions
	^ subscriptions! !

TestCase subclass: #GTExampleTestCase
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleTestCase class methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 11:24'!
isAbstract
	^ self name = #GTExampleTestCase! !

!GTExampleTestCase methodsFor: 'configuration' stamp: 'StefanReichhart 9/26/2015 10:17'!
packageNames
	^ {  }! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 10:49'!
packages: aCollection
	| packages |
	packages := OrderedCollection new.
	RPackageOrganizer default packages do: [ :package |
		aCollection do: [ :packageName |
			(package matchesPackagePattern: packageName) ifTrue: [ 
				packages add: package ] ] ].
	^ packages! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:11'!
run: aResult
	result := aResult.
	[ super run: aResult ] ensure: [ result := nil ]! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:12'!
runCase
	result ifNil: [ result := self classForTestResult new " fake for debug runner " ].
	super runCase. ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:56'!
runExample: example
	" this is not the testing tool we have in mind, but a temporary solution to integrate examples into the currently existing testing tools "
	| exampleResult |
	example ignoreTest ifTrue: [ ^ self ].
	self shouldnt: [ exampleResult := example run ] raise: Error.
	exampleResult isError 
		ifTrue: [ ^ self classForTestResult signalErrorWith: exampleResult gtDisplayString  ].
	exampleResult isFailure 
		ifTrue: [ ^ self classForTestResult signalFailureWith: exampleResult gtDisplayString  ] ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:58'!
runExampleContainers: aCollection
	| allExamples |
	(allExamples := aCollection gather: [ :container | container gtExamplesContained ])
		do: [ :example | self runExample: example ]
		displayingProgress: 'Running ' , allExamples size asString , ' Examples'! !

!GTExampleTestCase methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 11:17'!
testExamples
	" default testcase "
	self runExampleContainers: (self packages: self packageNames)! !

RBBlockLintRule subclass: #GTExampleNotDefinedRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleNotDefinedRule commentStamp: 'AndreiChis 1/22/2016 16:20' prior: 0!
Every class that defines a custom extension for GTSpotter or GTInspector should define also, at least, a GTExample.!

!GTExampleNotDefinedRule class methodsFor: 'testing' stamp: 'AndreiChis 1/22/2016 15:46'!
checksClass
	^ true! !

!GTExampleNotDefinedRule class methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:02'!
uniqueIdentifierName
	"This number should be unique and should change only when the rule completely change semantics"
	
	^'GTExampleNotDefinedRule'! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:56'!
basicCheck: aClass 
	^ (self containsCustomExtension: aClass) 
		and: [ (self containsGTExample: aClass) not ]! !

!GTExampleNotDefinedRule methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/22/2016 16:15'!
check: aClass forCriticsDo: aCriticBlock ifNone: alternativeBlock
	(self basicCheck: aClass)
		ifTrue:  [ aCriticBlock cull: (
			(ReMissingMethodCritic
				for: aClass
				by: self
				class: aClass class
				selector: #gtExample) beShouldBeImplemented )  ]
		ifFalse: alternativeBlock ! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:10'!
checkClass: aClass 
	(self basicCheck: aClass) ifTrue: [
		result
			addClass: aClass ]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:09'!
containsCustomExtension: aClass

	^ aClass methods anySatisfy: [:aMethod | 
		aMethod pragmas anySatisfy: [ :aPragma |
			aPragma keyword = GTInspector extensionsPragma or: [
				aPragma keyword = GTSpotter extensionsPragma ]]]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:59'!
containsGTExample: aClass

	^ aClass class methods anySatisfy: [:aMethod | 
			aMethod pragmas 
				anySatisfy: [ :pragma | pragma isGTExamplePragma ]]! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
group
	^ 'Design Flaws'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:00'!
name
	^ 'Class defines a custom GT extension but no example'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
resultClass
	^ RBClassEnvironment! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
severity
	^ #information! !

!ClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:45'!
gtExamplesFinder
	^ GTExampleFinderOLD new
		source: self gtExamplesSource;
		provider: self gtExamplesProvider;
		subjects: self gtExamplesSubjects;
		yourself! !

!Exception methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 7/13/2015 11:27'!
isExampleError
	^ false! !

Announcement subclass: #GTExampleAnnouncement
	instanceVariableNames: 'example'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

GTExampleAnnouncement subclass: #GTExampleAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleAnnouncement methodsFor: 'accessing' stamp: 'StefanReichhart 1/10/2016 11:06'!
example
	^ example! !

!GTExampleAnnouncement methodsFor: 'accessing' stamp: 'StefanReichhart 1/10/2016 11:06'!
example: anObject
	example := anObject! !

GTExampleAnnouncement subclass: #GTExampleModified
	instanceVariableNames: 'oldExample'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!GTExampleModified methodsFor: 'accessing' stamp: 'StefanReichhart 1/10/2016 11:07'!
oldExample
	^ oldExample! !

!GTExampleModified methodsFor: 'accessing' stamp: 'StefanReichhart 1/10/2016 11:07'!
oldExample: anObject
	oldExample := anObject! !

GTExampleAnnouncement subclass: #GTExampleRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-DEPRECATED'!

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:43'!
asGTExampleSubject
	^ GTExampleMethodSubject new 
		subjectClass: self methodClass;
		subjectSelector: self selector;
		yourself! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:37'!
gtExample
	self gtExamplesFactory exampleMethod: self do: [ :example | ^ example ].
	^ nil! !

!CompiledMethod methodsFor: '*GT-Examples-Convenience' stamp: 'StefanReichhart 5/29/2016 12:22'!
gtExampleValue
	^ self gtExample returnValue! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:38'!
gtExamples
	| examples |
	examples := OrderedCollection new.
	self methodClass theNonMetaClass gtExamplesFinder method: self to: examples.
	^ examples! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:32'!
isGTExampleMethod
	^ self pragmas anySatisfy: [ :each | each isGTExamplePragma ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/24/2016 12:17'!
valueWithReceiver: aReceiver possibleArguments: anArray 
	^ anArray size = self numArgs 
		ifTrue: [ self valueWithReceiver: aReceiver arguments: anArray  ]
		ifFalse: [ self valueWithReceiver: aReceiver arguments: (anArray copyFrom: 1 to: self numArgs) ]! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:13'!
gtExamples
	^ self gtExamplesFactory gtExamplesContained! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:52'!
gtExamplesDo: aBlock
	self gtExamplesFactory gtExamplesContainedDo: aBlock! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:58'!
gtExamplesFactory
	^ GTExampleFactory new
		source: self;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 17:48'!
gtExamplesFinder
	^ GTExampleFinder new
		source: self gtExamplesSource;
		provider: self gtExamplesProvider;
		subjects: self gtExamplesSubjects;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:46'!
gtExamplesProvider
	" by default let a class be the provider for examples "
	^ self! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:46'!
gtExamplesSource
	" by default let a class be the source for examples "
	^ self! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:25'!
gtExamplesSubjects
	^ {  }! !

!Error methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 7/3/2016 12:22'!
asGTExamplePragmaError
	^ GTExamplePragmaError new 
		signaledException: self;
		yourself! !

!Error methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 6/28/2015 17:09'!
isExamplesFailure
	^ true! !

Error subclass: #GTAbstractExampleError
	instanceVariableNames: 'signaledException'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

Error subclass: #GTExampleError
	instanceVariableNames: 'example signaledException'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

GTExampleError subclass: #GTExampleArgumentError
	instanceVariableNames: 'expected defined evaluated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:29'!
defined
	^ defined! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:29'!
defined: anObject
	defined := anObject! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:28'!
evaluated
	^ evaluated! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:28'!
evaluated: anObject
	evaluated := anObject! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:17'!
expected
	^ expected! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:17'!
expected: anObject
	expected := anObject! !

GTExampleError subclass: #GTExampleDependencyError
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 9/2/2015 22:02'!
exception
	^ exception! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 9/2/2015 22:02'!
exception: anObject
	exception := anObject! !

!GTExampleDependencyError methodsFor: 'testing' stamp: 'StefanReichhart 9/4/2015 15:40'!
isExampleError
	^ self exception 
		ifNotNil: [ self exception isExampleError ]
		ifNil: [ super isExampleError ]! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example
	^ example! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example: anObject
	example := anObject! !

!GTExampleError methodsFor: 'testing' stamp: 'StefanReichhart 7/13/2015 12:03'!
isExampleError
	^ true! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException
	^ signaledException! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException: anObject
	signaledException := anObject! !

GTExampleError subclass: #GTExamplePragmaError
	instanceVariableNames: 'pragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

GTExamplePragmaError subclass: #GTExampleDeclaredClassMissing
	instanceVariableNames: 'classNamed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed
	^ classNamed! !

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed: anObject
	classNamed := anObject! !

GTExamplePragmaError subclass: #GTExampleDeclaredMethodMissing
	instanceVariableNames: 'methodClass selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:52'!
methodClass
	^ methodClass! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:52'!
methodClass: anObject
	methodClass := anObject! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:51'!
selector
	^ selector! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:51'!
selector: anObject
	selector := anObject! !

!GTExamplePragmaError methodsFor: 'as yet unclassified' stamp: 'StefanReichhart 7/3/2016 12:22'!
asGTExamplePragmaError
	^ self! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma
	^ pragma! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma: anObject
	pragma := anObject! !

GTExamplePragmaError subclass: #GTExampleSubjectError
	instanceVariableNames: 'subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject
	^ subject! !

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject: anObject
	subject := anObject! !

GTExampleError subclass: #GTExampleRecursionDetected
	instanceVariableNames: 'resolved next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next
	^ next! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next: anObject
	next := anObject! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved
	^ resolved! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved: anObject
	resolved := anObject! !

Error subclass: #GTExampleExpectedExceptionNotRaised
	instanceVariableNames: 'expectedError exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception
	^ exception! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception: anObject
	exception := anObject! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError
	^ expectedError! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError: anObject
	expectedError := anObject! !

Error subclass: #GTExampleUnexpectedExceptionRaised
	instanceVariableNames: 'unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleUnexpectedExceptionRaised methodsFor: 'ui' stamp: 'StefanReichhart 4/10/2016 11:14'!
gtDisplayOn: aStream
	super gtDisplayOn: aStream.
	aStream nextPutAll: ' ('.
	self unexpectedError gtDisplayOn: aStream.
	aStream nextPutAll: ')'.! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError
	^ unexpectedError! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError: anObject
	unexpectedError := anObject! !

!SystemNavigation methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/2/2016 18:45'!
examplesReferencesTo: aLiteral
	^ GTExampleFinderOLD allGTExamples select: [ :each | 
			(each hasSubject: aLiteral)
				or: [ (each method hasLiteralThorough: aLiteral)
					or: [ (each source theNonMetaClass name asString = aLiteral name asString) ] ] ]! !

AbstractMethodIconAction subclass: #GTExampleNautilusAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-UI'!

!GTExampleNautilusAction class methodsFor: 'private' stamp: 'StefanReichhart 11/29/2015 11:50'!
iconForMethod: aCompiledMethod result: aResult
	aResult
		ifNil: [ ^ Smalltalk ui icons iconNamed: #testNotRunIcon ].
	aResult isSuccess
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testGreenIcon ].
	aResult isFailure
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testYellowIcon ].
	aResult isError
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testRedIcon ].
	^ Smalltalk ui icons iconNamed: #testNotRunIcon! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 9/4/2015 17:51'!
actionOrder
	"Return the priority of this action"
	
	^ 0! !

!GTExampleNautilusAction methodsFor: 'private' stamp: 'StefanReichhart 11/29/2015 11:50'!
iconForMethod: aCompiledMethod
	^ self class iconForMethod: aCompiledMethod result: (self resultForMethod: aCompiledMethod)! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 9/6/2015 16:58'!
isActionHandled
	^ method pragmas anySatisfy: [ :each | each isGTExamplePragma ]! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 9/4/2015 17:56'!
privateActionIcon
	"Return the icon for this action"
	^ IconicButton new
		target: self;
		actionSelector: #run:;
		arguments: { method };
		labelGraphic: (self iconForMethod: method);
		color: Color transparent;
		helpText: 'Run';
		extent: 12 @ 12;
		borderWidth: 0! !

!GTExampleNautilusAction methodsFor: 'private' stamp: 'StefanReichhart 11/29/2015 11:41'!
resultForMethod: aCompiledMethod
	^ aCompiledMethod gtExamples 
		ifNotEmpty: [ :examples | examples first resultCached ]
		ifEmpty: [ nil ]! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 12/5/2015 16:58'!
run: aCompiledMethod
	aCompiledMethod gtExamples do: [ :example | 
		example run isSuccess ifFalse: [ example debug ] ]! !

!TClass methodsFor: '*GT-Examples-UI' stamp: 'TudorGirba 1/26/2016 23:53'!
gtInspectorGTExamplesIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list 
		title: 'Examples';
		display: [ self gtExamples ];
		icon: [:each | each icon];
		format: [:each | String streamContents: [:s | each gtDisplaySummaryOn: s]] ;
		send: [ :each | each returnValue ];
		when: [ self gtExamples notEmpty ]! !

!TClass methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 9/6/2015 16:46'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamples ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!Class methodsFor: '*GT-Examples-UI' stamp: 'AliakseiSyrel 5/19/2016 15:03'!
gtInspectorGTExamplesIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list 
		title: 'Examples';
		display: [ self gtExamples ];
		icon: [:each | each icon];
		format: [:each | String streamContents: [:s | each gtDisplaySummaryOn: s]];
		in: [ :list | list sorted: [ :a :b | (list format cull: a) < (list format cull: b)] ];
		send: [ :each | each returnValue ];
		when: [ self gtExamples notEmpty ]! !

!RGMethodDefinition methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:44'!
asGTExampleSubject
	^ GTExampleMethodSubject new 
		subjectClass: self realClass;
		subjectSelector: self selector;
		yourself! !

!Pragma methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/9/2015 19:54'!
isGTExamplePragma
	<toBeDeprecated: #gtExample:>
	^ self keyword = #gtExample
		or: [ self keyword = #gtExample: ]! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 12:46'!
examplesClassMenu: aBuilder
	<contextMenu>
	<nautilusGlobalClassMenu>
	"^ GTExampleNautilus new
		onClass: aBuilder;
		yourself"! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 12:46'!
examplesMethodMenu: aBuilder
	<contextMenu>
	<nautilusGlobalMethodMenu>
	"^ GTExampleNautilus new
		onCompiledMethod: aBuilder;
		yourself"! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 12:46'!
examplesPackageMenu: aBuilder
	<contextMenu>
	<nautilusGlobalPackageMenu>
	"^ GTExampleNautilus new
		onPackage: aBuilder;
		yourself"! !
